@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2021 IRT Saint Exupéry, https://www.irt-saintexupery.com
 #
 # This program is free software; you can redistribute it and/or
@@ -13,23 +12,27 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-
 # Contributors:
 #    INITIAL AUTHORS - API and implementation and/or documentation
 #       :author: Pierre-Jean Barjhoux
 #       :author: Francois Gallard, integration and cleanup
 #    OTHER AUTHORS   - MACROSCOPIC CHANGES
-"""
-Implementation of the Lagrange multipliers
-******************************************
-"""
-from __future__ import division, unicode_literals
+"""Implementation of the Lagrange multipliers."""
+from __future__ import annotations
 
 import logging
 
-from numpy import arange, array, atleast_2d, concatenate, where, zeros
-from numpy.linalg import lstsq, matrix_rank, norm
+from numpy import arange
+from numpy import array
+from numpy import atleast_2d
+from numpy import concatenate
+from numpy import ndarray
+from numpy import where
+from numpy import zeros
+from numpy.linalg import matrix_rank
+from numpy.linalg import norm
 from scipy.optimize import linprog
+from scipy.optimize import nnls
 
 from gemseo.algos.design_space import DesignSpace
 from gemseo.algos.opt_problem import OptimizationProblem
@@ -38,9 +41,8 @@ from gemseo.third_party.prettytable import PrettyTable
 LOGGER = logging.getLogger(__name__)
 
 
-class LagrangeMultipliers(object):
-    r"""
-    Class that implements the computation of Lagrange Multipliers
+class LagrangeMultipliers:
+    r"""Class that implements the computation of Lagrange Multipliers.
 
     Denote :math:`x^\ast` an optimal solution of the optimization problem
     below.
@@ -74,7 +76,6 @@ class LagrangeMultipliers(object):
             &\lambda_{u,j}\ge0\text{ if }x^\ast_j=u_j,
             \text{ otherwise }\lambda_{u,j}=0.
         \end{aligned}\right.
-
     """
     LOWER_BOUNDS = "lower_bounds"
     UPPER_BOUNDS = "upper_bounds"
@@ -82,11 +83,11 @@ class LagrangeMultipliers(object):
     EQUALITY = "equality"
     CSTR_LABELS = [LOWER_BOUNDS, UPPER_BOUNDS, INEQUALITY, EQUALITY]
 
-    def __init__(self, opt_problem):
-        """Constructor.
-
-        :param opt_problem: optimization problem on which Lagrange multipliers
-            shall be computed
+    def __init__(self, opt_problem: OptimizationProblem) -> None:
+        """
+        Args:
+            opt_problem: The optimization problem
+                on which Lagrange multipliers shall be computed.
         """
         self._check_inputs(opt_problem)
         self.opt_problem = opt_problem
@@ -95,39 +96,48 @@ class LagrangeMultipliers(object):
         self.active_ineq_names = []
         self.active_eq_names = []
         self.lagrange_multipliers = None
-        self.__normalized = opt_problem.preprocess_options.get("normalize", False)
+        self.__normalized = opt_problem.preprocess_options.get(
+            "is_function_input_normalized", False
+        )
 
     @staticmethod
-    def _check_inputs(opt_problem):
-        """
-        Checks inputs : verify that opt_problem is an
-        instance of OptimizationProblem
+    def _check_inputs(opt_problem: OptimizationProblem) -> None:
+        """Verify that an problem is an instance of :class:`.OptimizationProblem`.
 
-        :param opt_problem: optimization problem on which Lagrange multipliers
-            shall be computed
+        Args:
+            opt_problem: The optimization problem
+                on which Lagrange multipliers shall be computed.
+
+        Raises:
+            ValueError: When the problem is not an :class:`.OptimizationProblem`
+                or when the problem was not solved.
         """
         if not isinstance(opt_problem, OptimizationProblem):
             raise ValueError(
-                "Argument of LagrangeMultiplier class"
-                + " has to be an instance of OptimizationProblem"
+                "LagrangeMultipliers must be initialized with an OptimizationProblem."
             )
         if opt_problem.solution is None:
-            raise ValueError("Optimization problem was not solved !")
+            raise ValueError("The optimization problem was not solved.")
 
-    def compute(self, x_vect, ineq_tolerance=1e-6, rcond=-1):
-        """Computes and returns the Lagrange multipliers, as a post-processing of the
-        optimal point.
+    def compute(
+        self, x_vect: ndarray, ineq_tolerance: float = 1e-6, rcond: float = -1
+    ) -> dict[str, tuple[list[str], ndarray]]:
+        """Compute the Lagrange multipliers, as a post-processing of the optimal point.
+
+        This solves:
 
-        This solves :
         (d ActiveConstraints)'               d Objective
         (-------------------)  . Lambda = -  -----------
         (d X                )                d X
 
-        :param x_vect: x point on which the multipliers shall be computed
-        :param ineq_tolerance: tolerance on inequality constraints
-        :param rcond: float, optional
-               Cut-off ratio for small singular values of the jacobian.
-               see sipy.linalg.lsq
+        Args:
+            x_vect: The optimal point on which the multipliers shall be computed.
+            ineq_tolerance: The tolerance on inequality constraints.
+            rcond: The cut-off ratio for small singular values of the Jacobian
+               (see scipy.linalg.lsq).
+
+        Returns:
+            The Lagrange multipliers.
         """
         LOGGER.info("Computation of Lagrange multipliers")
 
@@ -168,8 +178,7 @@ class LagrangeMultipliers(object):
         else:
             # If the linear optimization failed then obtain the Lagrange
             # multipliers as a solution of a least-square problem
-            mul, residuals, _, sval = lstsq(lhs, rhs, rcond=rcond)
-            LOGGER.info("Min singular values of jacobian = %s", str(sval.min()))
+            mul, residuals = nnls(lhs, rhs)
             LOGGER.info("Residuals norm = %s", str(norm(residuals)))
 
         # stores multipliers in a dictionary
@@ -177,13 +186,12 @@ class LagrangeMultipliers(object):
 
         return self.lagrange_multipliers
 
-    def _check_feasibility(self, x_vect):
-        """Checks that the given point is in the design space and satisfies all
+    def _check_feasibility(self, x_vect: ndarray) -> None:
+        """Check that the given point is in the design space and satisfies all
         constraints.
 
-        :param x_vect: point at which the Lagrange multipliers are to be
-            computed
-        :type x_vect: ndarray
+        Args:
+            x_vect: The point at which the Lagrange multipliers are to be computed.
         """
         problem = self.opt_problem
 
@@ -196,11 +204,20 @@ class LagrangeMultipliers(object):
         )
         feasible = problem.is_point_feasible(values)
         if not feasible:
-            LOGGER.warning("Infeasible point, Lagrange multipliers may not" " exist")
+            LOGGER.warning("Infeasible point, Lagrange multipliers may not exist.")
 
-    def _get_act_bound_jac(self, act_bounds):
-        """Returns the jacobian of active  bounds constraints sign is not taken into
-        account (matrix is made of 0 and 1)"""
+    def _get_act_bound_jac(self, act_bounds: dict[str, ndarray]):
+        """Return the Jacobian of the active bounds.
+
+        The constraints sign is not taken into account (matrix is made of 0 and 1).
+
+        Args:
+            act_bounds: The active bounds.
+
+        Returns:
+            The Jacobian of the active bounds
+            and the name of each component of each function.
+        """
         dspace = self.opt_problem.design_space
         x_dim = dspace.dimension
         dim_act = sum(len(where(bnd)[0]) for bnd in act_bounds.values())
@@ -220,9 +237,19 @@ class LagrangeMultipliers(object):
         act_b_names = indexed_varnames[act_array].tolist()
         return bnd_jac, act_b_names
 
-    def __get_act_ineq_jac(self, x_vect, ineq_tolerance=1e-6):
-        """Returns the jacobian of active inequality constraints defined by user in the
-        optimization problem."""
+    def __get_act_ineq_jac(
+        self, x_vect: ndarray, ineq_tolerance: float = 1e-6
+    ) -> tuple[ndarray, list[str]]:
+        """Return the Jacobian of the active inequality constraints.
+
+        Args:
+            x_vect: The point at which the Jacobian is computed.
+            ineq_tolerance: The tolerance for the inequality constraints.
+
+        Returns:
+            The Jacobian of the active inequality constraints
+            and the name of each component of each function.
+        """
         # retrieves the active functions and the indices :
         # a function is active if at least
         # one of its component (in case of multidimensional constraints) is
@@ -259,9 +286,16 @@ class LagrangeMultipliers(object):
             jac = None
         return jac, names
 
-    def _get_act_eq_jac(self, x_vect):
-        """Returns jacobian of active equality constraints defined by user in the
-        optimization problem."""
+    def _get_act_eq_jac(self, x_vect: ndarray) -> tuple[ndarray, list[str]]:
+        """Return The Jacobian of the active equality constraints.
+
+        Args:
+            x_vect: The point at which the Jacobian is computed.
+
+        Returns:
+            The Jacobian of the active equality constraints
+            and the name of each component of each function.
+        """
         eq_functions = self.opt_problem.get_eq_constraints()
         # loop on equality functions
         # NB: as the solution (x_vect) is supposed to be feasible,
@@ -289,16 +323,33 @@ class LagrangeMultipliers(object):
             jac = None
         return jac, names
 
-    def _get_obj_jac(self, x_vect):
-        """Returns objective jacobian."""
+    def _get_obj_jac(self, x_vect: ndarray) -> ndarray:
+        """Return the Jacobian of the objective.
+
+        Args:
+            x_vect: The point at which the Jacobian is computed.
+
+        Returns:
+            The Jacobian of the objective.
+        """
         if self.__normalized:
             x_vect = self.opt_problem.design_space.normalize_vect(x_vect)
 
         return self.opt_problem.objective.jac(x_vect)
 
-    def _get_jac_act(self, x_vect, ineq_tolerance=1e-6):
-        """Returns active constraints jacobian, and the name of each component of each
-        function."""
+    def _get_jac_act(
+        self, x_vect: ndarray, ineq_tolerance: float = 1e-6
+    ) -> tuple[ndarray, list[str]]:
+        """Return the Jacobian of the active constraints.
+
+        Args:
+            x_vect: The point at which the Jacobian is computed.
+            ineq_tolerance: The tolerance for the inequality constraints.
+
+        Returns:
+            The Jacobian of the active constraints
+            and the name of each component of each function.
+        """
         # Bounds jacobian
         dspace = self.opt_problem.design_space
         act_lb, act_ub = dspace.get_active_bounds(x_vect, tol=ineq_tolerance)
@@ -314,24 +365,31 @@ class LagrangeMultipliers(object):
         eq_jac, eq_names_act = self._get_act_eq_jac(x_vect)
         self.active_eq_names = eq_names_act
 
-        names_list = (
+        names = (
             self.active_lb_names
             + self.active_ub_names
             + self.active_ineq_names
             + eq_names_act
         )
-        jac_list = [lb_jac_act, ub_jac_act, ineq_jac, eq_jac]
-        jac_list = [jac for jac in jac_list if jac is not None]
-        if jac_list:
-            jac_act_arr = concatenate(jac_list, axis=0)
+        jacobians = [
+            jacobian
+            for jacobian in [lb_jac_act, ub_jac_act, ineq_jac, eq_jac]
+            if jacobian is not None
+        ]
+        if jacobians:
+            jac_act_arr = concatenate(jacobians, axis=0)
         else:
             # There no active constraint
             jac_act_arr = None
 
-        return jac_act_arr, names_list
+        return jac_act_arr, names
 
-    def _store_multipliers(self, multipliers):
-        """Stores multipliers in a dictionary."""
+    def _store_multipliers(self, multipliers: ndarray) -> None:
+        """Store the Lagrange multipliers in the attribute :attr:`lagrange_multipliers`.
+
+        Args:
+            multipliers: The Lagrange multipliers.
+        """
         lag = {}
 
         i_min = 0
@@ -377,7 +435,8 @@ class LagrangeMultipliers(object):
                     "%s !",
                     str(names_neg),
                 )
-        if self.active_eq_names:
+        n_act = len(self.active_eq_names)
+        if n_act > 0:
             lag[self.EQUALITY] = (
                 self.active_eq_names,
                 multipliers[i_min : i_min + n_act],
@@ -386,11 +445,11 @@ class LagrangeMultipliers(object):
 
         self.lagrange_multipliers = lag
 
-    def _initialize_multipliers(self):
+    def _initialize_multipliers(self) -> dict[str, dict[str, ndarray]]:
         """Initialize the Lagrange multipliers with zeros.
 
-        :returns: Lagrange multipliers
-        :rtype: dict(dict(float64))
+        Returns:
+            The Lagrange multipliers.
         """
         problem = self.opt_problem
         multipliers = dict()
@@ -416,11 +475,11 @@ class LagrangeMultipliers(object):
 
         return multipliers
 
-    def get_multipliers_arrays(self):
+    def get_multipliers_arrays(self) -> dict[str, dict[str, ndarray]]:
         """Return the Lagrange multipliers (zero and nonzero) as arrays.
 
-        :returns: Lagrange multipliers
-        :rtype: dict(dict(ndarray))
+        Returns:
+            The Lagrange multipliers.
         """
         problem = self.opt_problem
         design_space = problem.design_space
@@ -491,20 +550,20 @@ class LagrangeMultipliers(object):
         return mult_arrays
 
     @staticmethod
-    def _get_component_name(name, index):
+    def _get_component_name(name: str, index: int) -> str:
         """Return the name of a variable component.
 
-        :param name: the variable name
-        :type name: str
-        :param index: the component index
-        :type index: int
-        :returns: name of the variable component
-        :rtype: str
-        """
-        return name + DesignSpace.SEP + str(index)
+        Args:
+            name: The name of the variable.
+            index: The index of the component.
 
-    def _get_pretty_table(self):
-        """Displays Lagrange Multipliers."""
+        Returns:
+            The name of the variable component.
+        """
+        return f"{name}{DesignSpace.SEP}{index}"
+
+    def _get_pretty_table(self) -> PrettyTable:
+        """Display the Lagrange Multipliers."""
         table = PrettyTable(
             ["Constraint type", "Constraint name", "Lagrange Multiplier"]
         )
@@ -515,8 +574,5 @@ class LagrangeMultipliers(object):
 
         return table
 
-    def __str__(self, *args, **kwargs):
-        """Textual representation of the design space."""
-        desc = "Lagrange multipliers : "
-        desc += "\n" + str(self._get_pretty_table().get_string())
-        return desc
+    def __str__(self, *args, **kwargs) -> str:
+        return f"Lagrange multipliers:\n{self._get_pretty_table().get_string()}"
