@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2021 IRT Saint Exupéry, https://www.irt-saintexupery.com
 #
 # This program is free software; you can redistribute it and/or
@@ -13,7 +12,6 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-
 # Contributors:
 #    INITIAL AUTHORS - initial API and implementation and/or initial
 #                           documentation
@@ -21,20 +19,31 @@
 #    OTHER AUTHORS   - MACROSCOPIC CHANGES
 #         Francois Gallard : refactoring for v1, May 2016
 """scipy.optimize optimization library wrapper."""
-from __future__ import division, unicode_literals
+from __future__ import annotations
 
 import logging
-from typing import Any, Dict, Optional, Union
+from dataclasses import dataclass
+from typing import Any
 
-from numpy import isfinite, ndarray, real
+from numpy import isfinite
+from numpy import ndarray
+from numpy import real
 from scipy import optimize
 
+from gemseo.algos.opt.opt_lib import OptimizationAlgorithmDescription
 from gemseo.algos.opt.opt_lib import OptimizationLibrary
 from gemseo.algos.opt_result import OptimizationResult
 
 LOGGER = logging.getLogger(__name__)
 
 
+@dataclass
+class SciPyAlgorithmDescription(OptimizationAlgorithmDescription):
+    """The description of an optimization algorithm from the SciPy library."""
+
+    library_name: str = "SciPy"
+
+
 class ScipyOpt(OptimizationLibrary):
     """Scipy optimization library interface.
 
@@ -51,6 +60,8 @@ class ScipyOpt(OptimizationLibrary):
         OptimizationLibrary.PG_TOL: "gtol",
     }
 
+    LIBRARY_NAME = "SciPy"
+
     def __init__(self):
         """Constructor.
 
@@ -61,67 +72,69 @@ class ScipyOpt(OptimizationLibrary):
         - does it handle equality constraints
         - does it handle inequality constraints
         """
-        super(ScipyOpt, self).__init__()
+        super().__init__()
         doc = "https://docs.scipy.org/doc/scipy/reference/"
-        self.lib_dict = {
-            "SLSQP": {
-                self.INTERNAL_NAME: "SLSQP",
-                self.REQUIRE_GRAD: True,
-                self.POSITIVE_CONSTRAINTS: True,
-                self.HANDLE_EQ_CONS: True,
-                self.HANDLE_INEQ_CONS: True,
-                self.DESCRIPTION: "Sequential Least-Squares Quadratic "
-                "Programming (SLSQP) implemented in "
-                "the SciPy library",
-                self.WEBSITE: doc + "optimize.minimize-slsqp.html",
-            },
-            "L-BFGS-B": {
-                self.INTERNAL_NAME: "L-BFGS-B",
-                self.REQUIRE_GRAD: True,
-                self.HANDLE_EQ_CONS: False,
-                self.HANDLE_INEQ_CONS: False,
-                self.DESCRIPTION: "Limited-memory BFGS algorithm "
-                "implemented in SciPy library",
-                self.WEBSITE: doc + "generated/scipy.optimize.fmin_l_bfgs_b.html",
-            },
-            "TNC": {
-                self.INTERNAL_NAME: "TNC",
-                self.REQUIRE_GRAD: True,
-                self.HANDLE_EQ_CONS: False,
-                self.HANDLE_INEQ_CONS: False,
-                self.DESCRIPTION: "Truncated Newton (TNC) algorithm "
-                "implemented in SciPy library",
-                self.WEBSITE: doc + "optimize.minimize-tnc.html",
-            },
+        self.descriptions = {
+            "SLSQP": SciPyAlgorithmDescription(
+                algorithm_name="SLSQP",
+                description=(
+                    "Sequential Least-Squares Quadratic Programming (SLSQP) "
+                    "implemented in the SciPy library"
+                ),
+                handle_equality_constraints=True,
+                handle_inequality_constraints=True,
+                internal_algorithm_name="SLSQP",
+                require_gradient=True,
+                positive_constraints=True,
+                website=f"{doc}optimize.minimize-slsqp.html",
+            ),
+            "L-BFGS-B": SciPyAlgorithmDescription(
+                algorithm_name="L-BFGS-B",
+                description=(
+                    "Limited-memory BFGS algorithm implemented in SciPy library"
+                ),
+                internal_algorithm_name="L-BFGS-B",
+                require_gradient=True,
+                website=f"{doc}generated/scipy.optimize.fmin_l_bfgs_b.html",
+            ),
+            "TNC": SciPyAlgorithmDescription(
+                algorithm_name="TNC",
+                description=(
+                    "Truncated Newton (TNC) algorithm implemented in SciPy library"
+                ),
+                internal_algorithm_name="TNC",
+                require_gradient=True,
+                website=f"{doc}optimize.minimize-tnc.html",
+            ),
         }
 
     def _get_options(
         self,
-        max_iter=999,  # type: int
-        ftol_rel=1e-9,  # type: float
-        ftol_abs=1e-9,  # type: float
-        xtol_rel=1e-9,  # type: float
-        xtol_abs=1e-9,  # type: float
-        max_ls_step_size=0.0,  # type: float
-        max_ls_step_nb=20,  # type: int
-        max_fun_eval=999,  # type: int
-        max_time=0,  # type: float
-        pg_tol=1e-5,  # type: float
-        disp=0,  # type: int
-        maxCGit=-1,  # type: int # noqa: N803
-        eta=-1.0,  # type: float
-        factr=1e7,  # type: float
-        maxcor=20,  # type: int
-        normalize_design_space=True,  # type: int
-        eq_tolerance=1e-2,  # type: float
-        ineq_tolerance=1e-4,  # type: float
-        stepmx=0.0,  # type: float
-        minfev=0.0,  # type: float
-        scale=None,  # type: Optional[float]
-        rescale=-1,  # type: float
-        offset=None,  # type: Optional[float]
-        **kwargs  # type: Any
-    ):  # type: (...) -> Dict[str, Any]
+        max_iter: int = 999,
+        ftol_rel: float = 1e-9,
+        ftol_abs: float = 1e-9,
+        xtol_rel: float = 1e-9,
+        xtol_abs: float = 1e-9,
+        max_ls_step_size: float = 0.0,
+        max_ls_step_nb: int = 20,
+        max_fun_eval: int = 999,
+        max_time: float = 0,
+        pg_tol: float = 1e-5,
+        disp: int = 0,
+        maxCGit: int = -1,  # noqa: N803
+        eta: float = -1.0,
+        factr: float = 1e7,
+        maxcor: int = 20,
+        normalize_design_space: int = True,
+        eq_tolerance: float = 1e-2,
+        ineq_tolerance: float = 1e-4,
+        stepmx: float = 0.0,
+        minfev: float = 0.0,
+        scale: float | None = None,
+        rescale: float = -1,
+        offset: float | None = None,
+        **kwargs: Any,
+    ) -> dict[str, Any]:
         r"""Set the options default values.
 
         To get the best and up to date information about algorithms options,
@@ -195,13 +208,11 @@ class ScipyOpt(OptimizationLibrary):
             scale=scale,
             rescale=rescale,
             offset=offset,
-            **kwargs
+            **kwargs,
         )
         return popts
 
-    def _run(
-        self, **options  # type: Any
-    ):  # type: (...) -> OptimizationResult
+    def _run(self, **options: Any) -> OptimizationResult:
         """Run the algorithm, to be overloaded by subclasses.
 
         Args:
@@ -220,8 +231,8 @@ class ScipyOpt(OptimizationLibrary):
         bounds = list(zip(l_b, u_b))
 
         def real_part_fun(
-            x,  # type: ndarray
-        ):  # type: (...) -> Union[int, float]
+            x: ndarray,
+        ) -> int | float:
             """Wrap the function and return the real part.
 
             Args:
