@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2021 IRT Saint Exupéry, https://www.irt-saintexupery.com
 #
 # This program is free software; you can redistribute it and/or
@@ -13,28 +12,35 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-
 # Contributors:
 #    INITIAL AUTHORS - API and implementation and/or documentation
 #        :author: Francois Gallard, Charlie Vanaret
 #    OTHER AUTHORS   - MACROSCOPIC CHANGES
-"""Base class for all Multi-disciplinary Design Analyses (MDA).."""
-from __future__ import division, unicode_literals
+"""Base class for all Multi-disciplinary Design Analyses (MDA)."""
+from __future__ import annotations
 
 import logging
 from multiprocessing import cpu_count
-from typing import Any, Iterable, List, Mapping, Optional, Sequence, Set, Tuple, Union
+from pathlib import Path
+from typing import Any
+from typing import Iterable
+from typing import Mapping
+from typing import Sequence
 
 import matplotlib.pyplot as plt
 from matplotlib.ticker import MaxNLocator
-from numpy import array, concatenate, ndarray
+from numpy import array
+from numpy import concatenate
+from numpy import ndarray
 from numpy.linalg import norm
 
-from gemseo.core.coupling_structure import DependencyGraph, MDOCouplingStructure
+from gemseo.core.coupling_structure import DependencyGraph
+from gemseo.core.coupling_structure import MDOCouplingStructure
 from gemseo.core.discipline import MDODiscipline
-from gemseo.core.execution_sequence import ExecutionSequenceFactory, LoopExecSequence
+from gemseo.core.execution_sequence import ExecutionSequenceFactory
+from gemseo.core.execution_sequence import LoopExecSequence
 from gemseo.core.jacobian_assembly import JacobianAssembly
-from gemseo.utils.py23_compat import Path
+from gemseo.utils.matplotlib_figure import save_show_figure
 
 LOGGER = logging.getLogger(__name__)
 
@@ -64,23 +70,81 @@ class MDA(MDODiscipline):
         "matrix_type",
         "use_lu_fact",
         "linear_solver_tolerance",
+        "_scale_residuals_with_coupling_size",
+        "_scale_residuals_with_first_norm",
+        "_current_iter",
     )
 
+    activate_cache = True
+
+    tolerance: float
+    """The tolerance of the iterative direct coupling solver"""
+
+    linear_solver: str
+    """The name of the linear solver."""
+
+    linear_solver_tolerance: float
+    """The tolerance of the linear solver in the adjoint equation."""
+
+    linear_solver_options: dict[str, Any]
+    """The options of the linear solver."""
+
+    max_mda_iter: int
+    """The maximum iterations number for the MDA algorithm."""
+
+    disciplines: Sequence[MDODiscipline]
+    """The disciplines from which to compute the MDA."""
+
+    coupling_structure: MDOCouplingStructure
+    """The coupling structure to be used by the MDA."""
+
+    assembly: JacobianAssembly
+
+    residual_history: list
+    """The history of MDA residuals."""
+
+    reset_history_each_run: bool
+    """Whether to reset the history of MDA residuals before each run."""
+
+    warm_start: bool
+    """Whether the second iteration and ongoing start from the previous solution."""
+
+    norm0: float | None
+    """The reference residual, if any."""
+
+    normed_residual: float
+    """The normed residual."""
+
+    strong_couplings: list[str]
+    """The names of the strong coupling variables."""
+
+    all_couplings: list[str]
+    """The names of the coupling variables."""
+
+    matrix_type: str
+    """The type of the matrix."""
+
+    use_lu_fact: bool
+    """Whether to store a LU factorization of the matrix."""
+
+    lin_cache_tol_fact: float
+    """The tolerance factor to cache the Jacobian."""
+
     def __init__(
         self,
-        disciplines,  # type: Sequence[MDODiscipline]
-        max_mda_iter=10,  # type: int
-        name=None,  # type: Optional[str]
-        grammar_type=MDODiscipline.JSON_GRAMMAR_TYPE,  # type: str
-        tolerance=1e-6,  # type: float
-        linear_solver_tolerance=1e-12,  # type: float
-        warm_start=False,  # type: bool
-        use_lu_fact=False,  # type: bool
-        coupling_structure=None,  # type: Optional[MDOCouplingStructure]
-        log_convergence=False,  # type: bool
-        linear_solver="DEFAULT",  # type: str
-        linear_solver_options=None,  # type: Mapping[str,Any]
-    ):  # type: (...) -> None
+        disciplines: Sequence[MDODiscipline],
+        max_mda_iter: int = 10,
+        name: str | None = None,
+        grammar_type: str = MDODiscipline.JSON_GRAMMAR_TYPE,
+        tolerance: float = 1e-6,
+        linear_solver_tolerance: float = 1e-12,
+        warm_start: bool = False,
+        use_lu_fact: bool = False,
+        coupling_structure: MDOCouplingStructure | None = None,
+        log_convergence: bool = False,
+        linear_solver: str = "DEFAULT",
+        linear_solver_options: Mapping[str, Any] = None,
+    ) -> None:
         """
         Args:
             disciplines: The disciplines from which to compute the MDA.
@@ -88,7 +152,8 @@ class MDA(MDODiscipline):
             name: The name to be given to the MDA.
                 If None, use the name of the class.
             grammar_type: The type of the input and output grammars,
-                either :attr:`JSON_GRAMMAR_TYPE` or :attr:`SIMPLE_GRAMMAR_TYPE`.
+                either :attr:`.MDODiscipline.JSON_GRAMMAR_TYPE`
+                or :attr:`.MDODiscipline.SIMPLE_GRAMMAR_TYPE`.
             tolerance: The tolerance of the iterative direct coupling solver;
                 the norm of the current residuals divided by initial residuals norm
                 shall be lower than the tolerance to stop iterating.
@@ -106,7 +171,7 @@ class MDA(MDODiscipline):
             linear_solver: The name of the linear solver.
             linear_solver_options: The options passed to the linear solver factory.
         """
-        super(MDA, self).__init__(name, grammar_type=grammar_type)
+        super().__init__(name, grammar_type=grammar_type)
         self.tolerance = tolerance
         self.linear_solver = linear_solver
         self.linear_solver_tolerance = linear_solver_tolerance
@@ -121,18 +186,21 @@ class MDA(MDODiscipline):
         self.residual_history = []
         self.reset_history_each_run = False
         self.warm_start = warm_start
+        self._scale_residuals_with_coupling_size = False
+        self._scale_residuals_with_first_norm = True
 
         # Don't erase coupling values before calling _compute_jacobian
 
         self._linearize_on_last_state = True
         self.norm0 = None
+        self._current_iter = 0
         self.normed_residual = 1.0
-        self.strong_couplings = self.coupling_structure.strong_couplings()
-        self.all_couplings = self.coupling_structure.get_all_couplings()
+        self.strong_couplings = self.coupling_structure.strong_couplings
+        self.all_couplings = self.coupling_structure.all_couplings
         self._input_couplings = []
         self.matrix_type = JacobianAssembly.SPARSE
         self.use_lu_fact = use_lu_fact
-        # By default dont use an approximate cache for linearization
+        # By default don't use an approximate cache for linearization
         self.lin_cache_tol_fact = 0.0
 
         self._initialize_grammars()
@@ -141,28 +209,28 @@ class MDA(MDODiscipline):
         self._check_couplings_types()
         self._log_convergence = log_convergence
 
-    def _initialize_grammars(self):  # type: (...) -> None
+    def _initialize_grammars(self) -> None:
         """Define all the inputs and outputs of the MDA.
 
         Add all the outputs of all the disciplines to the outputs.
         """
         for discipline in self.disciplines:
-            self.input_grammar.update_from(discipline.input_grammar)
-            self.output_grammar.update_from(discipline.output_grammar)
+            self.input_grammar.update(discipline.input_grammar)
+            self.output_grammar.update(discipline.output_grammar)
 
     @property
-    def log_convergence(self):  # type: (...) -> bool
+    def log_convergence(self) -> bool:
         """Whether to log the MDA convergence."""
         return self._log_convergence
 
     @log_convergence.setter
     def log_convergence(
         self,
-        value,  # type: bool
-    ):  # type: (...) -> None
+        value: bool,
+    ) -> None:
         self._log_convergence = value
 
-    def __check_linear_solver_options(self):  # type: (...) -> None
+    def __check_linear_solver_options(self) -> None:
         """Check the linear solver options.
 
         The linear solver tolerance cannot be set
@@ -179,17 +247,13 @@ class MDA(MDODiscipline):
             )
             raise ValueError(msg)
 
-    def _check_consistency(self):  # type: (...) -> None
+    def _check_consistency(self) -> None:
         """Check if there are not more than one equation per variable.
 
-        For instance if a strong coupling is not also a self coupling.
-
-        Raises:
-            ValueError:
-                * If there are too many coupling constraints.
-                * If outputs are defined multiple times.
+        For instance if a strong coupling is not also a self coupling, or if outputs are
+        defined multiple times.
         """
-        strong_c_disc = self.coupling_structure.strongly_coupled_disciplines(
+        strong_c_disc = self.coupling_structure.get_strongly_coupled_disciplines(
             add_self_coupled=False
         )
         also_strong = [
@@ -199,8 +263,8 @@ class MDA(MDODiscipline):
         ]
         if also_strong:
             for disc in also_strong:
-                in_outs = set(disc.get_input_data_names()) & set(
-                    disc.get_output_data_names()
+                in_outs = sorted(
+                    set(disc.get_input_data_names()) & set(disc.get_output_data_names())
                 )
                 LOGGER.warning(
                     "Self coupling variables in discipline %s are: %s.",
@@ -208,13 +272,12 @@ class MDA(MDODiscipline):
                     in_outs,
                 )
 
-            also_strong_n = [disc.name for disc in also_strong]
-            raise ValueError(
-                "Too many coupling constraints; "
-                "the following disciplines are self coupled "
-                "and also strongly coupled with other disciplines: {}.".format(
-                    also_strong_n
-                )
+            also_strong_n = sorted(disc.name for disc in also_strong)
+            LOGGER.warning(
+                "The following disciplines contain self-couplings and strong couplings:"
+                " %s. This is not a problem as long as their self-coupling variables "
+                "are not strongly coupled to another discipline.",
+                also_strong_n,
             )
 
         all_outs = {}
@@ -226,23 +289,24 @@ class MDA(MDODiscipline):
                 all_outs[out] = disc
 
         if multiple_outs:
-            raise ValueError(
-                "Outputs are defined multiple times: {}.".format(multiple_outs)
+            LOGGER.warning(
+                "The following outputs are defined multiple times: %s.",
+                sorted(multiple_outs),
             )
 
-    def _compute_input_couplings(self):  # type: (...) -> None
+    def _compute_input_couplings(self) -> None:
         """Compute the strong couplings that are inputs of the MDA."""
         input_couplings = set(self.strong_couplings) & set(self.get_input_data_names())
         self._input_couplings = list(input_couplings)
 
-    def _current_input_couplings(self):  # type: (...) -> ndarray
+    def _current_input_couplings(self) -> ndarray:
         """Return the current values of the input coupling variables."""
         input_couplings = list(iter(self.get_outputs_by_name(self._input_couplings)))
         if not input_couplings:
             return array([])
         return concatenate(input_couplings)
 
-    def _current_strong_couplings(self):  # type: (...) -> ndarray
+    def _current_strong_couplings(self) -> ndarray:
         """Return the current values of the strong coupling variables."""
         couplings = list(iter(self.get_outputs_by_name(self.strong_couplings)))
         if not couplings:
@@ -251,15 +315,15 @@ class MDA(MDODiscipline):
 
     def _retrieve_diff_inouts(
         self,
-        force_all=False,  # type: bool
-    ):  # type: (...) -> Tuple[Union[Set[str],List[str]],Union[Set[str],List[str]]]
+        force_all: bool = False,
+    ) -> tuple[set[str] | list[str], set[str] | list[str]]:
         """Return the names of the inputs and outputs involved in the differentiation.
 
         Args:
             force_all: Whether to differentiate all outputs with respect to all inputs.
                 If `False`,
-                differentiate the :attr:`_differentiated_outputs`
-                with respect to the :attr:`_differentiated_inputs`.
+                differentiate the :attr:`.MDODiscipline._differentiated_outputs`
+                with respect to the :attr:`.MDODiscipline._differentiated_inputs`.
 
         Returns:
             The inputs according to which to differentiate,
@@ -279,9 +343,9 @@ class MDA(MDODiscipline):
 
         return MDODiscipline._retrieve_diff_inouts(self, False)
 
-    def _couplings_warm_start(self):  # type: (...) -> None
+    def _couplings_warm_start(self) -> None:
         """Load the previous couplings values to local data."""
-        cached_outputs = self.cache.get_last_cached_outputs()
+        cached_outputs = self.cache.last_entry.outputs
         if not cached_outputs:
             return
         for input_name in self._input_couplings:
@@ -289,7 +353,7 @@ class MDA(MDODiscipline):
             if input_value is not None:
                 self.local_data[input_name] = input_value
 
-    def _set_default_inputs(self):  # type: (...) -> None
+    def _set_default_inputs(self) -> None:
         """Set the default input values of the MDA from the disciplines ones."""
         self.default_inputs = {}
         mda_input_names = self.get_input_data_names()
@@ -300,42 +364,41 @@ class MDA(MDODiscipline):
                         input_name
                     ]
 
-    def _check_couplings_types(self):  # type: (...) -> None
+    def _check_couplings_types(self) -> None:
         """Check that the coupling variables are of type array in the grammars.
 
         Raises:
-            ValueError: When at least one of the coupling variables is not an array.
+            TypeError: When at least one of the coupling variables is not an array.
         """
         not_arrays = []
         for discipline in self.disciplines:
             for grammar in (discipline.input_grammar, discipline.output_grammar):
                 for coupling in self.all_couplings:
-                    exists = grammar.is_data_name_existing(coupling)
-                    if exists and not grammar.is_type_array(coupling):
+                    if coupling in grammar and not grammar.is_array(coupling):
                         not_arrays.append(coupling)
 
-        not_arrays = sorted(set(not_arrays))
         if not_arrays:
-            raise ValueError(
-                "The coupling variables {} must be of type array.".format(not_arrays)
+            not_arrays = sorted(set(not_arrays))
+            raise TypeError(
+                f"The coupling variables {not_arrays} must be of type array."
             )
 
-    def reset_disciplines_statuses(self):  # type: (...) -> None
+    def reset_disciplines_statuses(self) -> None:
         """Reset all the statuses of the disciplines."""
         for discipline in self.disciplines:
             discipline.reset_statuses_for_run()
 
-    def reset_statuses_for_run(self):  # type: (...) -> None
+    def reset_statuses_for_run(self) -> None:
         MDODiscipline.reset_statuses_for_run(self)
         self.reset_disciplines_statuses()
 
-    def get_expected_workflow(self):  # type: (...) ->LoopExecSequence
+    def get_expected_workflow(self) -> LoopExecSequence:
         disc_exec_seq = ExecutionSequenceFactory.serial(self.disciplines)
         return ExecutionSequenceFactory.loop(self, disc_exec_seq)
 
     def get_expected_dataflow(
         self,
-    ):  # type: (...) -> List[Tuple[MDODiscipline,MDODiscipline,List[str]]]
+    ) -> list[tuple[MDODiscipline, MDODiscipline, list[str]]]:
         all_disc = [self]
         all_disc.extend(self.disciplines)
         graph = DependencyGraph(all_disc)
@@ -344,9 +407,9 @@ class MDA(MDODiscipline):
 
     def _compute_jacobian(
         self,
-        inputs=None,  # type: Optional[Iterable[str]]
-        outputs=None,  # type: Optional[Iterable[str]]
-    ):  # type: (...) -> None
+        inputs: Iterable[str] | None = None,
+        outputs: Iterable[str] | None = None,
+    ) -> None:
         # Do not re execute disciplines if inputs error is beyond self tol
         # Apply a safety factor on this (mda is a loop, inputs
         # of first discipline
@@ -355,11 +418,20 @@ class MDA(MDODiscipline):
         exec_cache_tol = self.lin_cache_tol_fact * self.tolerance
         force_no_exec = exec_cache_tol != 0.0
         self.__check_linear_solver_options()
+        residual_variables = {}
+        for disc in self.disciplines:
+            residual_variables.update(disc.residual_variables)
+
+        couplings_adjoint = list(
+            set(self.all_couplings)
+            - set(residual_variables.keys())
+            - set(residual_variables.values())
+        )
         self.jac = self.assembly.total_derivatives(
             self.local_data,
             outputs,
             inputs,
-            self.all_couplings,
+            couplings_adjoint,
             tol=self.linear_solver_tolerance,
             mode=self.linearization_mode,
             matrix_type=self.matrix_type,
@@ -367,76 +439,84 @@ class MDA(MDODiscipline):
             exec_cache_tol=exec_cache_tol,
             force_no_exec=force_no_exec,
             linear_solver=self.linear_solver,
-            **self.linear_solver_options
+            residual_variables=residual_variables,
+            **self.linear_solver_options,
         )
 
-    # fixed point methods
     def _compute_residual(
         self,
-        current_couplings,  # type: ndarray
-        new_couplings,  # type: ndarray
-        current_iter,  # type: int
-        first=False,  # type: bool
-        store_it=True,  # type: bool
-        log_normed_residual=False,  # type: bool
-    ):  # type: (...) -> ndarray
+        current_couplings: ndarray,
+        new_couplings: ndarray,
+        store_it: bool = True,
+        log_normed_residual: bool = False,
+    ) -> ndarray:
         """Compute the residual on the inputs of the MDA.
 
         Args:
             current_couplings: The values of the couplings before the execution.
             new_couplings: The values of the couplings after the execution.
-            current_iter: The current iteration of the fixed-point method.
-            first: Whether it is the first residual of the fixed-point method.
             store_it: Whether to store the normed residual.
             log_normed_residual: Whether to log the normed residual.
 
         Returns:
             The normed residual.
         """
-        if first and self.reset_history_each_run:
+        if self._current_iter == 0 and self.reset_history_each_run:
             self.residual_history = []
 
         normed_residual = norm((current_couplings - new_couplings).real)
-        if self.norm0 is None:
-            self.norm0 = normed_residual
-        if self.norm0 == 0:
-            self.norm0 = 1
-        self.normed_residual = normed_residual / self.norm0
+
+        if self._scale_residuals_with_first_norm:
+            if self._scale_residuals_with_coupling_size:
+                if self.norm0 is not None:
+                    self.normed_residual = normed_residual / self.norm0
+                else:
+                    self.normed_residual = normed_residual / new_couplings.size**0.5
+            else:
+                if self.norm0 is None:
+                    self.norm0 = normed_residual
+                if self.norm0 == 0:
+                    self.norm0 = 1
+                self.normed_residual = normed_residual / self.norm0
+        else:
+            self.normed_residual = normed_residual
+
         if log_normed_residual:
             LOGGER.info(
                 "%s running... Normed residual = %s (iter. %s)",
                 self.name,
-                "{:.2e}".format(self.normed_residual),
-                current_iter,
+                f"{self.normed_residual:.2e}",
+                self._current_iter,
             )
 
         if store_it:
-            self.residual_history.append((self.normed_residual, current_iter))
+            self.residual_history.append(self.normed_residual)
+            self._current_iter += 1
         return self.normed_residual
 
     def check_jacobian(
         self,
-        input_data=None,  # type: Optional[Mapping[str,ndarray]]
-        derr_approx=FINITE_DIFFERENCES,  # type: str
-        step=1e-7,  # type: float
-        threshold=1e-8,  # type: float
-        linearization_mode="auto",  # type: str
-        inputs=None,  # type: Optional[Iterable[str]]
-        outputs=None,  # type: Optional[Iterable[str]]
-        parallel=False,  # type: bool
-        n_processes=N_CPUS,  # type: int
-        use_threading=False,  # type: bool
-        wait_time_between_fork=0,  # type: int
-        auto_set_step=False,  # type: bool
-        plot_result=False,  # type: bool
-        file_path="jacobian_errors.pdf",  # type: Union[str,Path]
-        show=False,  # type: bool
-        figsize_x=10,  # type: float
-        figsize_y=10,  # type: float
+        input_data: Mapping[str, ndarray] | None = None,
+        derr_approx: str = FINITE_DIFFERENCES,
+        step: float = 1e-7,
+        threshold: float = 1e-8,
+        linearization_mode: str = "auto",
+        inputs: Iterable[str] | None = None,
+        outputs: Iterable[str] | None = None,
+        parallel: bool = False,
+        n_processes: int = N_CPUS,
+        use_threading: bool = False,
+        wait_time_between_fork: int = 0,
+        auto_set_step: bool = False,
+        plot_result: bool = False,
+        file_path: str | Path = "jacobian_errors.pdf",
+        show: bool = False,
+        fig_size_x: float = 10,
+        fig_size_y: float = 10,
         reference_jacobian_path=None,
         save_reference_jacobian=False,
         indices=None,
-    ):  # type: (...) -> bool
+    ) -> bool:
         """Check if the analytical Jacobian is correct with respect to a reference one.
 
         If `reference_jacobian_path` is not `None`
@@ -467,7 +547,9 @@ class MDA(MDODiscipline):
             step: The step
                 for finite differences or complex step differentiation methods.
             parallel: Whether to execute the MDA in parallel.
-            n_processes: The maximum number of processors on which to run.
+            n_processes: The maximum simultaneous number of threads,
+                if ``use_threading`` is True, or processes otherwise,
+                used to parallelize the execution.
             use_threading: Whether to use threads instead of processes
                 to parallelize the execution;
                 multiprocessing will copy (serialize) all the disciplines,
@@ -483,8 +565,8 @@ class MDA(MDODiscipline):
                 comparing the exact and approximated Jacobians.
             file_path: The path to the output file if `plot_result` is `True`.
             show: Whether to open the figure.
-            figsize_x: The *x* size of the figure in inches.
-            figsize_y: The *y* size of the figure in inches.
+            fig_size_x: The *x* size of the figure in inches.
+            fig_size_y: The *y* size of the figure in inches.
             reference_jacobian_path: The path of the reference Jacobian file.
             save_reference_jacobian: Whether to save the reference Jacobian.
             indices: The indices of the inputs and outputs
@@ -499,7 +581,7 @@ class MDA(MDODiscipline):
                 If a variable name is missing, consider all its components.
                 If None, consider all the components of all the ``inputs`` and ``outputs``.
 
-        Return:
+        Returns:
             Whether the passed Jacobian is correct.
         """
         # Strong couplings are not linearized
@@ -519,7 +601,7 @@ class MDA(MDODiscipline):
             if str_cpl in outputs:
                 outputs.remove(str_cpl)
 
-        return super(MDA, self).check_jacobian(
+        return super().check_jacobian(
             input_data=input_data,
             derr_approx=derr_approx,
             step=step,
@@ -535,21 +617,19 @@ class MDA(MDODiscipline):
             plot_result=plot_result,
             file_path=file_path,
             show=show,
-            figsize_x=figsize_x,
-            figsize_y=figsize_y,
+            fig_size_x=fig_size_x,
+            fig_size_y=fig_size_y,
             reference_jacobian_path=reference_jacobian_path,
             save_reference_jacobian=save_reference_jacobian,
             indices=indices,
         )
 
-    def _warn_convergence_criteria(
-        self,
-        current_iter,  # type: int
-    ):  # type: (...) -> Tuple[bool,bool]
-        """Log a warning if max_iter is reached and if max residuals is above tolerance.
+    def execute(self, input_data: Mapping[str, Any] | None = None) -> dict[str, Any]:
+        self._current_iter = 0
+        return super().execute(input_data=input_data)
 
-        Args:
-            current_iter: The current iteration of the MDA.
+    def _warn_convergence_criteria(self) -> tuple[bool, bool]:
+        """Log a warning if max_iter is reached and if max residuals is above tolerance.
 
         Returns:
             * Whether the normed residual is lower than the tolerance.
@@ -557,7 +637,7 @@ class MDA(MDODiscipline):
         """
 
         residual_is_small = self.normed_residual <= self.tolerance
-        max_iter_is_reached = self.max_mda_iter <= current_iter
+        max_iter_is_reached = self.max_mda_iter <= self._current_iter
         if max_iter_is_reached and not residual_is_small:
             msg = (
                 "%s has reached its maximum number of iterations "
@@ -566,27 +646,16 @@ class MDA(MDODiscipline):
             LOGGER.warning(msg, self.name, self.normed_residual, self.tolerance)
         return residual_is_small, max_iter_is_reached
 
-    def _termination(
-        self,
-        current_iter,  # type: int
-    ):  # type: (...) -> bool
-        """Termination criterion.
-
-        Args:
-            current_iter: The current iteration of the fixed point method.
-
-        Returns:
-            Whether to stop the MDA algorithm.
-        """
-        residual_is_small, max_iter_is_reached = self._warn_convergence_criteria(
-            current_iter
-        )
+    @property
+    def _stop_criterion_is_reached(self) -> bool:
+        """Whether a stop criterion is reached."""
+        residual_is_small, max_iter_is_reached = self._warn_convergence_criteria()
         return residual_is_small or max_iter_is_reached
 
     def _set_cache_tol(
         self,
-        cache_tol,  # type: float
-    ):  # type: (...) -> None
+        cache_tol: float,
+    ) -> None:
         """Set to the cache input tolerance.
 
         To be overloaded by subclasses.
@@ -594,19 +663,40 @@ class MDA(MDODiscipline):
         Args:
             cache_tol: The cache tolerance.
         """
-        super(MDA, self)._set_cache_tol(cache_tol)
+        super()._set_cache_tol(cache_tol)
         for disc in self.disciplines:
             disc.cache_tol = cache_tol or 0.0
 
+    def set_residuals_scaling_options(
+        self,
+        scale_residuals_with_coupling_size: bool = False,
+        scale_residuals_with_first_norm: bool = True,
+    ) -> None:
+        """Set the options for the residuals scaling.
+
+        Args:
+            scale_residuals_with_coupling_size: Whether to activate the scaling of the MDA
+                residuals by the number of coupling variables.
+                This divides the residuals obtained by the norm of the difference
+                between iterates by the square root of the coupling vector size.
+
+            scale_residuals_with_first_norm: Whether to scale the residuals by the first
+                residual norm, except if `:attr:.norm0` is set by the user.
+                If `:attr:.norm0` is set to a value, it deactivates
+                the residuals scaling by the design variables size.
+        """
+        self._scale_residuals_with_coupling_size = scale_residuals_with_coupling_size
+        self._scale_residuals_with_first_norm = scale_residuals_with_first_norm
+
     def plot_residual_history(
         self,
-        show=False,  # type: bool
-        save=True,  # type: bool
-        n_iterations=None,  # type: Optional[int]
-        logscale=None,  # type: Optional[Tuple[int,int]]
-        filename=None,  # type: Optional[str]
-        figsize=(50, 10),  # type: Tuple[int,int]
-    ):  # type: (...) -> None
+        show: bool = False,
+        save: bool = True,
+        n_iterations: int | None = None,
+        logscale: tuple[int, int] | None = None,
+        filename: str | None = None,
+        fig_size: tuple[float, float] = (50.0, 10.0),
+    ) -> None:
         """Generate a plot of the residual history.
 
         All residuals are stored in the history;
@@ -622,28 +712,41 @@ class MDA(MDODiscipline):
                 If None, do not change the limits of the *y* axis.
             filename: The name of the file to save the figure.
                 If None, use "{mda.name}_residual_history.pdf".
-            figsize: The *x* and *y* sizes of the figure in inches.
+            fig_size: The width and height of the figure in inches, e.g. `(w, h)`.
+
+        Returns:
+            The figure, to be customized if not closed.
         """
-        fig = plt.figure(figsize=figsize)
+        fig = plt.figure()
         fig_ax = fig.add_subplot(1, 1, 1)
 
-        # split list of couples
-        residual = [res for (res, _) in self.residual_history]
+        history_length = len(self.residual_history)
+        n_iterations = n_iterations or history_length
+
+        if n_iterations > history_length:
+            msg = (
+                "Requested %s iterations but the residual history contains only %s, "
+                "plotting all the residual history."
+            )
+            LOGGER.info(msg, n_iterations, history_length)
+            n_iterations = history_length
+
         # red dot for first iteration
-        colors = [
-            "red" if current_iter == 1 else "black"
-            for (_, current_iter) in self.residual_history
-        ]
+        colors = ["black"] * n_iterations
+        colors[0] = "red"
 
         fig_ax.scatter(
-            list(range(len(residual))), residual, s=20, color=colors, zorder=2
+            list(range(n_iterations)),
+            self.residual_history[:n_iterations],
+            s=20,
+            color=colors,
+            zorder=2,
+        )
+        fig_ax.plot(
+            self.residual_history[:n_iterations], linestyle="-", c="k", zorder=1
         )
-        fig_ax.plot(residual, linestyle="-", c="k", zorder=1)
         fig_ax.axhline(y=self.tolerance, c="blue", linewidth=0.5, zorder=0)
-        fig_ax.set_title("{}: residual plot".format(self.name))
-
-        if n_iterations is None:
-            n_iterations = len(self.residual_history)
+        fig_ax.set_title(f"{self.name}: residual plot")
 
         plt.yscale("log")
         plt.xlabel(r"iterations", fontsize=14)
@@ -655,10 +758,8 @@ class MDA(MDODiscipline):
 
         if save:
             if filename is None:
-                filename = "{}_residual_history.pdf".format(self.name)
-            plt.savefig(filename, bbox_inches="tight")
+                filename = f"{self.name}_residual_history.pdf"
 
-        if show:
-            plt.show()
-        else:
-            plt.close(fig)
+        save_show_figure(fig, show, filename, fig_size)
+
+        return fig
