@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2021 IRT Saint Exupéry, https://www.irt-saintexupery.com
 #
 # This program is free software; you can redistribute it and/or
@@ -13,7 +12,6 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-
 # Contributors:
 #    INITIAL AUTHORS - API and implementation and/or documentation
 #        :author: Charlie Vanaret, Francois Gallard
@@ -28,11 +26,13 @@ Root finding methods include:
 Each of these methods is implemented by a class in this module.
 Both inherit from a common abstract cache.
 """
-from __future__ import division, unicode_literals
+from __future__ import annotations
 
 import logging
 from copy import deepcopy
-from typing import Any, Dict, List, Mapping, Optional, Sequence, Union
+from typing import Any
+from typing import Mapping
+from typing import Sequence
 
 from numpy import ndarray
 from numpy.linalg import norm
@@ -41,7 +41,8 @@ from scipy.optimize import root
 from gemseo.core.coupling_structure import MDOCouplingStructure
 from gemseo.core.discipline import MDODiscipline
 from gemseo.mda.mda import MDA
-from gemseo.utils.data_conversion import DataConversion
+from gemseo.utils.data_conversion import concatenate_dict_of_arrays_to_array
+from gemseo.utils.data_conversion import update_dict_of_arrays_from_array
 
 # from gemseo.core.parallel_execution import DisciplinesParallelExecution
 LOGGER = logging.getLogger(__name__)
@@ -54,22 +55,22 @@ class MDARoot(MDA):
 
     def __init__(
         self,
-        disciplines,  # type: Sequence[MDODiscipline]
-        max_mda_iter=10,  # type: int
-        name=None,  # type: Optional[str]
-        grammar_type=MDODiscipline.JSON_GRAMMAR_TYPE,  # type: str
-        tolerance=1e-6,  # type: float
-        linear_solver_tolerance=1e-12,  # type: float
-        warm_start=False,  # type: bool
-        use_lu_fact=False,  # type: bool
-        coupling_structure=None,  # type: Optional[MDOCouplingStructure]
-        log_convergence=False,  # type: bool
-        linear_solver="DEFAULT",  # type: str
-        linear_solver_options=None,  # type: Mapping[str,Any]
-    ):  # type: (...) -> None
+        disciplines: Sequence[MDODiscipline],
+        max_mda_iter: int = 10,
+        name: str | None = None,
+        grammar_type: str = MDODiscipline.JSON_GRAMMAR_TYPE,
+        tolerance: float = 1e-6,
+        linear_solver_tolerance: float = 1e-12,
+        warm_start: bool = False,
+        use_lu_fact: bool = False,
+        coupling_structure: MDOCouplingStructure | None = None,
+        log_convergence: bool = False,
+        linear_solver: str = "DEFAULT",
+        linear_solver_options: Mapping[str, Any] = None,
+    ) -> None:
         self.tolerance = 1e-6
         self.max_mda_iter = 10
-        super(MDARoot, self).__init__(
+        super().__init__(
             disciplines,
             max_mda_iter=max_mda_iter,
             name=name,
@@ -94,15 +95,15 @@ class MDARoot(MDA):
         #    self.parallel_execution = None
         # ==================================================================
 
-    def _initialize_grammars(self):  # type: (...) -> None
+    def _initialize_grammars(self) -> None:
         for disciplines in self.disciplines:
-            self.input_grammar.update_from(disciplines.input_grammar)
-            self.output_grammar.update_from(disciplines.output_grammar)
+            self.input_grammar.update(disciplines.input_grammar)
+            self.output_grammar.update(disciplines.output_grammar)
 
     def execute_all_disciplines(
         self,
-        input_local_data,  # type: Mapping[str,ndarray]
-    ):  # type: (...) -> None
+        input_local_data: Mapping[str, ndarray],
+    ) -> None:
         """Execute all self.disciplines.
 
         Args:
@@ -145,25 +146,25 @@ class MDANewtonRaphson(MDARoot):
 
     def __init__(
         self,
-        disciplines,  # type: Sequence[MDODiscipline]
-        max_mda_iter=10,  # type: int
-        relax_factor=0.99,  # type: float
-        name=None,  # type: Optional[str]
-        grammar_type=MDODiscipline.JSON_GRAMMAR_TYPE,  # type: str
-        linear_solver="DEFAULT",  # type: str
-        tolerance=1e-6,  # type: float
-        linear_solver_tolerance=1e-12,  # type: float
-        warm_start=False,  # type: bool
-        use_lu_fact=False,  # type: bool
-        coupling_structure=None,  # type: Optional[MDOCouplingStructure]
-        log_convergence=False,  # type:bool
-        linear_solver_options=None,  # type: Mapping[str,Any]
+        disciplines: Sequence[MDODiscipline],
+        max_mda_iter: int = 10,
+        relax_factor: float = 0.99,
+        name: str | None = None,
+        grammar_type: str = MDODiscipline.JSON_GRAMMAR_TYPE,
+        linear_solver: str = "DEFAULT",
+        tolerance: float = 1e-6,
+        linear_solver_tolerance: float = 1e-12,
+        warm_start: bool = False,
+        use_lu_fact: bool = False,
+        coupling_structure: MDOCouplingStructure | None = None,
+        log_convergence: bool = False,
+        linear_solver_options: Mapping[str, Any] = None,
     ):
         """
         Args:
             relax_factor: The relaxation factor in the Newton step.
         """
-        super(MDANewtonRaphson, self).__init__(
+        super().__init__(
             disciplines,
             max_mda_iter=max_mda_iter,
             name=name,
@@ -182,8 +183,8 @@ class MDANewtonRaphson(MDARoot):
 
     @staticmethod
     def __check_relax_factor(
-        relax_factor,  # type: float
-    ):  # type:(...) -> float
+        relax_factor: float,
+    ) -> float:
         """Check that the relaxation factor in the Newton step is in (0, 1].
 
         Args:
@@ -196,7 +197,7 @@ class MDANewtonRaphson(MDARoot):
             )
         return relax_factor
 
-    def _newton_step(self):  # type: (...) -> None
+    def _newton_step(self) -> None:
         """Execute the full Newton step.
 
         Compute the increment :math:`-[dR/dW]^{-1}.R` and run the disciplines.
@@ -207,7 +208,7 @@ class MDANewtonRaphson(MDARoot):
             self.relax_factor,
             self.linear_solver,
             matrix_type=self.matrix_type,
-            **self.linear_solver_options
+            **self.linear_solver_options,
         )
         # update current solution with Newton step
         exec_data = deepcopy(self.local_data)
@@ -216,7 +217,7 @@ class MDANewtonRaphson(MDARoot):
         self.reset_disciplines_statuses()
         self.execute_all_disciplines(exec_data)
 
-    def _run(self):  # type: (...) -> None
+    def _run(self) -> None:
         if self.warm_start:
             self._couplings_warm_start()
         # execute the disciplines
@@ -225,27 +226,20 @@ class MDANewtonRaphson(MDARoot):
         self.execute_all_disciplines(self.local_data)
         new_couplings = self._current_input_couplings()
 
-        # store initial residual
-        current_iter = 1
         self._compute_residual(
             current_couplings,
             new_couplings,
-            current_iter,
-            first=True,
             log_normed_residual=self.log_convergence,
         )
         current_couplings = new_couplings
 
-        while not self._termination(current_iter):
+        while not self._stop_criterion_is_reached:
             self._newton_step()
             new_couplings = self._current_input_couplings()
 
-            # store current residual
-            current_iter += 1
             self._compute_residual(
                 current_couplings,
                 new_couplings,
-                current_iter,
                 log_normed_residual=self.log_convergence,
             )
             current_couplings = new_couplings
@@ -311,30 +305,30 @@ class MDAQuasiNewton(MDARoot):
 
     def __init__(
         self,
-        disciplines,  # type: Sequence[MDODiscipline]
-        max_mda_iter=10,  # type: int
-        name=None,  # type: Optional[str]
-        grammar_type=MDODiscipline.JSON_GRAMMAR_TYPE,  # type: str
-        method=HYBRID,  # type: str
-        use_gradient=False,  # type: bool
-        tolerance=1e-6,  # type: float
-        linear_solver_tolerance=1e-12,  # type: float
-        warm_start=False,  # type: bool
-        use_lu_fact=False,  # type: bool
-        coupling_structure=None,  # type: Optional[MDOCouplingStructure]
-        linear_solver="DEFAULT",  # type: str
-        linear_solver_options=None,  # type: Mapping[str,Any]
+        disciplines: Sequence[MDODiscipline],
+        max_mda_iter: int = 10,
+        name: str | None = None,
+        grammar_type: str = MDODiscipline.JSON_GRAMMAR_TYPE,
+        method: str = HYBRID,
+        use_gradient: bool = False,
+        tolerance: float = 1e-6,
+        linear_solver_tolerance: float = 1e-12,
+        warm_start: bool = False,
+        use_lu_fact: bool = False,
+        coupling_structure: MDOCouplingStructure | None = None,
+        linear_solver: str = "DEFAULT",
+        linear_solver_options: Mapping[str, Any] = None,
     ):
         """
         Args:
             method: The name of the method in scipy root finding,
-                among :attr:`QUASI_NEWTON_METHODS`.
+                among :attr:`.QUASI_NEWTON_METHODS`.
             use_gradient: Whether to use the analytic gradient of the discipline.
 
         Raises:
             ValueError: If the method is not a valid quasi-Newton method.
         """
-        super(MDAQuasiNewton, self).__init__(
+        super().__init__(
             disciplines,
             max_mda_iter=max_mda_iter,
             name=name,
@@ -348,14 +342,14 @@ class MDAQuasiNewton(MDARoot):
             coupling_structure=coupling_structure,
         )
         if method not in self.QUASI_NEWTON_METHODS:
-            msg = "Method '{}' is not a valid quasi-Newton method.".format(method)
+            msg = f"Method '{method}' is not a valid quasi-Newton method."
             raise ValueError(msg)
         self.method = method
         self.use_gradient = use_gradient
         self.local_residual_history = []
         self.last_outputs = None  # used for computing the residual history
 
-    def _solver_options(self):  # type: (...) -> Dict[str,Union[float,int]]
+    def _solver_options(self) -> dict[str, float | int]:
         """Determine options for the solver, based on the resolution method."""
         options = {}
         if self.method in [
@@ -380,11 +374,11 @@ class MDAQuasiNewton(MDARoot):
             options["maxfev"] = self.max_mda_iter
         return options
 
-    def _methods_with_callback(self):  # type: (...) -> List[str]
+    def _methods_with_callback(self) -> list[str]:
         """Determine whether resolution method accepts a callback function."""
         return [self.BROYDEN1, self.BROYDEN2]
 
-    def _run(self):  # type: (...) -> Dict[str,ndarray]
+    def _run(self) -> dict[str, ndarray]:
         if self.warm_start:
             self._couplings_warm_start()
         self.reset_disciplines_statuses()
@@ -404,8 +398,8 @@ class MDAQuasiNewton(MDARoot):
         self.current_iter = 0
 
         def fun(
-            x_vect,  # type: ndarray
-        ):  # type: (...) -> ndarray
+            x_vect: ndarray,
+        ) -> ndarray:
             """Evaluate all residuals, possibly in parallel.
 
             Args:
@@ -413,7 +407,7 @@ class MDAQuasiNewton(MDARoot):
             """
             self.current_iter += 1
             # transform input vector into a dict
-            input_values = DataConversion.update_dict_from_array(
+            input_values = update_dict_of_arrays_from_array(
                 self.local_data, couplings, x_vect
             )
             # compute all residuals
@@ -438,15 +432,15 @@ class MDAQuasiNewton(MDARoot):
             # linearize the residuals
 
             def jacobian(
-                x_vect,  # type: ndarray
-            ):  # type: (...) -> ndarray
+                x_vect: ndarray,
+            ) -> ndarray:
                 """Linearize all residuals.
 
                 Args:
                     x_vect: The value of the design variables.
                 """
                 # transform input vector into a dict
-                input_values = DataConversion.update_dict_from_array(
+                input_values = update_dict_of_arrays_from_array(
                     self.local_data, couplings, x_vect
                 )
                 # linearize all residuals
@@ -468,7 +462,7 @@ class MDAQuasiNewton(MDARoot):
             jac = jacobian
 
         # initial solution
-        y_0 = DataConversion.dict_to_array(self.local_data, couplings).real
+        y_0 = concatenate_dict_of_arrays_to_array(self.local_data, couplings).real
         # callback function to retrieve the residual at iteration k
         norm_0 = norm(y_0.real)
         if self.reset_history_each_run:
@@ -479,17 +473,18 @@ class MDAQuasiNewton(MDARoot):
         if self.method in self._methods_with_callback():
 
             def callback(
-                y_k,  # type: ndarray
+                y_k: ndarray,
                 _,
-            ):  # type: (...) -> None
+            ) -> None:
                 """Store the current residual in the history.
 
                 Args:
                     y_k: The coupling variables.
                     _: ignored
                 """
-                delta = norm((y_k - self.last_outputs).real) / norm_0
-                self.residual_history.append((delta, 0))  # iter number?
+                self.residual_history.append(
+                    norm((y_k - self.last_outputs).real) / norm_0
+                )
                 self.last_outputs = y_k
 
         else:
@@ -499,10 +494,10 @@ class MDAQuasiNewton(MDARoot):
         y_opt = root(
             fun, x0=y_0, method=self.method, jac=jac, callback=callback, options=options
         )
-        self._warn_convergence_criteria(self.current_iter)
+        self._warn_convergence_criteria()
 
         # transform optimal vector into a dict
-        self.local_data = DataConversion.update_dict_from_array(
+        self.local_data = update_dict_of_arrays_from_array(
             self.local_data, couplings, y_opt.x
         )
         return self.local_data
