@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2021 IRT Saint Exupéry, https://www.irt-saintexupery.com
 #
 # This program is free software; you can redistribute it and/or
@@ -18,7 +17,6 @@
 #                           documentation
 #        :author: Matthias De Lozzo
 #    OTHER AUTHORS   - MACROSCOPIC CHANGES
-
 """Abstract class for the computation and analysis of sensitivity indices.
 
 The purpose of a sensitivity analysis is to
@@ -30,41 +28,43 @@ a :class:`.ParameterSpace` describing the uncertain parameters
 and options associated with a particular concrete class
 inheriting from :class:`.SensitivityAnalysis` which is an abstract one.
 """
-
-from __future__ import division, unicode_literals
+from __future__ import annotations
 
 import logging
 from copy import deepcopy
-from typing import (
-    Dict,
-    Iterable,
-    List,
-    Mapping,
-    NoReturn,
-    Optional,
-    Sequence,
-    Tuple,
-    Union,
-)
+from pathlib import Path
+from typing import Any
+from typing import Collection
+from typing import Dict
+from typing import Iterable
+from typing import List
+from typing import Mapping
+from typing import NoReturn
+from typing import Sequence
+from typing import Tuple
+from typing import Union
 
-import six
-from custom_inherit import DocInheritMeta
+from docstring_inheritance import GoogleDocstringInheritanceMeta
 from matplotlib.figure import Figure
-from numpy import array, linspace, ndarray, vstack
+from numpy import array
+from numpy import linspace
+from numpy import ndarray
+from numpy import vstack
 
 from gemseo.algos.doe.doe_lib import DOELibraryOptionType
 from gemseo.algos.parameter_space import ParameterSpace
 from gemseo.core.dataset import Dataset
 from gemseo.core.discipline import MDODiscipline
 from gemseo.core.doe_scenario import DOEScenario
+from gemseo.disciplines.utils import get_all_outputs
 from gemseo.post.dataset.bars import BarPlot
 from gemseo.post.dataset.curves import Curves
 from gemseo.post.dataset.dataset_plot import DatasetPlotPropertyType
 from gemseo.post.dataset.radar_chart import RadarChart
 from gemseo.post.dataset.surfaces import Surfaces
-from gemseo.utils.file_path_manager import FilePathManager, FileType
+from gemseo.utils.file_path_manager import FilePathManager
+from gemseo.utils.file_path_manager import FileType
 from gemseo.utils.matplotlib_figure import save_show_figure
-from gemseo.utils.py23_compat import Path
 
 LOGGER = logging.getLogger(__name__)
 
@@ -72,14 +72,7 @@ OutputsType = Union[str, Tuple[str, int], Sequence[Union[str, Tuple[str, int]]]]
 IndicesType = Dict[str, List[Dict[str, ndarray]]]
 
 
-@six.add_metaclass(
-    DocInheritMeta(
-        abstract_base_class=True,
-        style="google_with_merge",
-        include_special_methods=True,
-    )
-)
-class SensitivityAnalysis(object):
+class SensitivityAnalysis(metaclass=GoogleDocstringInheritanceMeta):
     """Sensitivity analysis.
 
     The :class:`.SensitivityAnalysis` class provides both
@@ -95,85 +88,144 @@ class SensitivityAnalysis(object):
 
     Lastly, the :meth:`.SensitivityAnalysis.plot_comparison` method allows
     to compare the current :class:`.SensitivityAnalysis` with another one.
-
-    Attributes:
-        default_output (list(str)): The default outputs of interest.
-        dataset (Dataset): The dataset containing the discipline evaluations.
     """
 
+    default_output: list[str]
+    """The default outputs of interest."""
+
+    dataset: Dataset
+    """The dataset containing the discipline evaluations."""
+
     DEFAULT_DRIVER = None
 
     def __init__(
         self,
-        discipline,  # type: MDODiscipline
-        parameter_space,  # type: ParameterSpace
-        n_samples,  # type: Optional[int]
-        algo=None,  # type: Optional[str]
-        algo_options=None,  # type: Optional[Mapping[str,DOELibraryOptionType]]
-    ):  # type: (...) -> None  # noqa: D205,D212,D415
-        """
+        disciplines: Collection[MDODiscipline],
+        parameter_space: ParameterSpace,
+        n_samples: int | None = None,
+        output_names: Iterable[str] = None,
+        algo: str | None = None,
+        algo_options: Mapping[str, DOELibraryOptionType] | None = None,
+        formulation: str = "MDF",
+        **formulation_options: Any,
+    ) -> None:
+        """.. # noqa: D205,D212,D415
         Args:
-            discipline: A discipline.
+            disciplines: The discipline or disciplines to use for the analysis.
             parameter_space: A parameter space.
             n_samples: A number of samples.
-                If None, the number of samples is computed by the algorithm.
+                If ``None``, the number of samples is computed by the algorithm.
+            output_names: The disciplines' outputs to be considered for the analysis.
+                If ``None``, use all the outputs.
             algo: The name of the DOE algorithm.
-                If None, use the :attr:`DEFAULT_DRIVER`.
+                If ``None``, use the :attr:`.SensitivityAnalysis.DEFAULT_DRIVER`.
             algo_options: The options of the DOE algorithm.
+            formulation: The name of the :class:`.MDOFormulation` to sample the disciplines.
+            **formulation_options: The options of the :class:`.MDOFormulation`.
         """
+        disciplines = list(disciplines)
+
         self._algo_name = algo or self.DEFAULT_DRIVER
-        self.default_output = list(discipline.get_output_data_names())
+        self.default_output = output_names or get_all_outputs(disciplines)
         algo_options = algo_options or {}
-        self.dataset = self.__sample_discipline(
-            discipline, parameter_space, n_samples, **algo_options
-        )
+        formulation_options = formulation_options or {}
+
+        self.dataset = self.__sample_disciplines(
+            disciplines,
+            parameter_space,
+            n_samples,
+            algo_options,
+            formulation,
+            **formulation_options,
+        ).export_to_dataset(opt_naming=False)
+
         self._main_method = self.__class__.__name__
         default_name = FilePathManager.to_snake_case(self.__class__.__name__)
         self._file_path_manager = FilePathManager(
             FileType.FIGURE, default_name=default_name
         )
 
-    def __sample_discipline(
+    def __sample_disciplines(
         self,
-        discipline,  # type: MDODiscipline
-        parameter_space,  # type: ParameterSpace,
-        n_samples,  # type: Optional[int]
-        **options  # type: DOELibraryOptionType
-    ):  # type: (...) -> Dataset
-        """Sample the discipline and return the dataset.
+        disciplines: Sequence[MDODiscipline],
+        parameter_space: ParameterSpace,
+        n_samples: int | None,
+        algo_options: Mapping[str, DOELibraryOptionType],
+        formulation: str,
+        **formulation_options: Any,
+    ) -> DOEScenario:
+        """Sample the disciplines and return the scenario after evaluation.
 
         Args:
-            discipline: A discipline.
+            disciplines: The disciplines to sample.
             parameter_space: A parameter space.
             n_samples: A number of samples.
-                If None, the number of samples is computed by the algorithm.
-            **options: The options for the DOE algorithm.
+                If ``None``, the number of samples is computed by the algorithm.
+            algo_options: The options for the DOE algorithm.
+            formulation: The name of the :class:`.MDOFormulation` to sample the disciplines.
+            **formulation_options: The options of the :class:`.MDOFormulation`.
 
         Returns:
-            The evaluations of the discipline.
+            The DOE scenario after evaluation.
         """
-        discipline.set_cache_policy(discipline.MEMORY_FULL_CACHE)
-        scenario = DOEScenario(
-            [discipline], "DisciplinaryOpt", self.default_output[0], parameter_space
+        scenario = self._create_scenario(
+            disciplines,
+            self.default_output[0],
+            formulation,
+            formulation_options,
+            parameter_space,
         )
+
         scenario.execute(
             {
                 "algo": self._algo_name,
                 "n_samples": n_samples,
-                "algo_options": options,
+                "algo_options": algo_options,
             }
         )
-        inputs = parameter_space.variables_names
-        return discipline.cache.export_to_dataset(inputs_names=inputs)
+        return scenario
+
+    def _create_scenario(
+        self,
+        disciplines: Iterable[MDODiscipline],
+        objective_name: str,
+        formulation: str,
+        formulation_options: Mapping[str, Any],
+        parameter_space: ParameterSpace,
+    ) -> DOEScenario:
+        """Create a DOE scenario to sample the disciplines.
+
+        Args:
+            disciplines: The disciplines to sample.
+            objective_name: The name of the objective for the DOE.
+            formulation: The name of the :class:`.MDOFormulation` to sample the disciplines.
+            formulation_options: The options of the :class:`.MDOFormulation`.
+            parameter_space: A parameter space.
+
+        Returns:
+            The DOE scenario to be used to sample the disciplines.
+        """
+        scenario = DOEScenario(
+            disciplines,
+            formulation,
+            objective_name,
+            parameter_space,
+            **formulation_options,
+        )
+        for discipline in disciplines:
+            for output_name in discipline.get_output_data_names():
+                if output_name != objective_name:
+                    scenario.add_observable(output_name)
+        return scenario
 
     @property
-    def inputs_names(self):  # type: (...) -> List[str]
+    def inputs_names(self) -> list[str]:
         """The names of the inputs."""
         return self.dataset.get_names(self.dataset.INPUT_GROUP)
 
     def compute_indices(
-        self, outputs=None  # type: Optional[Sequence[str]]
-    ):  # type: (...) -> Dict[str,IndicesType]
+        self, outputs: Sequence[str] | None = None
+    ) -> dict[str, IndicesType]:
         """Compute the sensitivity indices.
 
         Args:
@@ -201,7 +253,7 @@ class SensitivityAnalysis(object):
         raise NotImplementedError
 
     @property
-    def indices(self):  # type: (...) -> Dict[str,IndicesType]
+    def indices(self) -> dict[str, IndicesType]:
         """The sensitivity indices.
 
         With the following structure:
@@ -221,19 +273,19 @@ class SensitivityAnalysis(object):
         raise NotImplementedError
 
     @property
-    def main_method(self):  # type: (...) -> str
+    def main_method(self) -> str:
         """The name of the main method."""
         return self._main_method
 
     @main_method.setter
     def main_method(
         self,
-        name,  # type: str
-    ):  # type: (...) -> NoReturn
+        name: str,
+    ) -> NoReturn:
         raise NotImplementedError("You cannot change the main method.")
 
     @property
-    def main_indices(self):  # type: (...) -> IndicesType
+    def main_indices(self) -> IndicesType:
         """The main sensitivity indices.
 
         With the following structure:
@@ -252,8 +304,8 @@ class SensitivityAnalysis(object):
 
     def _outputs_to_tuples(
         self,
-        outputs,  # type: OutputsType
-    ):  # type: (...) -> List[Tuple[str,int]]
+        outputs: OutputsType,
+    ) -> list[tuple[str, int]]:
         """Convert the outputs to a list of tuple(str,int).
 
         Args:
@@ -285,9 +337,7 @@ class SensitivityAnalysis(object):
         ]
         return [item for sublist in result for item in sublist]
 
-    def sort_parameters(
-        self, output  # type: Union[str,Tuple[str,int]]
-    ):  # type: (...) -> List[str]
+    def sort_parameters(self, output: str | tuple[str, int]) -> list[str]:
         """Return the parameters sorted in descending order.
 
         Args:
@@ -314,14 +364,14 @@ class SensitivityAnalysis(object):
 
     def plot(
         self,
-        output,  # type: Union[str,Tuple[str,int]]
-        inputs=None,  # type: Optional[Iterable[str]]
-        title=None,  # type: Optional[str]
-        save=True,  # type: bool
-        show=False,  # type: bool
-        file_path=None,  # type: Optional[Union[str,Path]]
-        file_format=None,  # type: Optional[str]
-    ):  # type: (...) -> None
+        output: str | tuple[str, int],
+        inputs: Iterable[str] | None = None,
+        title: str | None = None,
+        save: bool = True,
+        show: bool = False,
+        file_path: str | Path | None = None,
+        file_format: str | None = None,
+    ) -> None:
         """Plot the sensitivity indices.
 
         Args:
@@ -345,19 +395,19 @@ class SensitivityAnalysis(object):
 
     def plot_field(
         self,
-        output,  # type: Union[str,Tuple[str,int]]
-        mesh=None,  # type: Optional[ndarray]
-        inputs=None,  # type: Optional[Iterable[str]]
-        standardize=False,  # type: bool
-        title=None,  # type: Optional[str]
-        save=True,  # type: bool
-        show=False,  # type: bool
-        file_path=None,  # type: Optional[Union[str,Path]]
-        directory_path=None,  # type: Optional[Union[str,Path]]
-        file_name=None,  # type: Optional[str]
-        file_format=None,  # type: Optional[str]
-        properties=None,  # type: Mapping[str,DatasetPlotPropertyType]
-    ):  # type: (...) -> Union[Curves,Surfaces]
+        output: str | tuple[str, int],
+        mesh: ndarray | None = None,
+        inputs: Iterable[str] | None = None,
+        standardize: bool = False,
+        title: str | None = None,
+        save: bool = True,
+        show: bool = False,
+        file_path: str | Path | None = None,
+        directory_path: str | Path | None = None,
+        file_name: str | None = None,
+        file_format: str | None = None,
+        properties: Mapping[str, DatasetPlotPropertyType] = None,
+    ) -> Curves | Surfaces:
         """Plot the sensitivity indices related to a 1D or 2D functional output.
 
         The output is considered as a 1D or 2D functional variable,
@@ -366,11 +416,12 @@ class SensitivityAnalysis(object):
         Args:
             output: The output
                 for which to display sensitivity indices,
-                either a name or a tuple of the form (name, component).
+                either a name or a tuple of the form (name, component)
+                where (name, component) is used to sort the inputs.
                 If name, its first component is considered.
             mesh: The mesh on which the p-length output
-                is represented. Either a (1, p) array for a 1D functional output
-                or a (2, p) array for a 2D one. If None, assume a 1D functional output.
+                is represented. Either a p-length array for a 1D functional output
+                or a (p, 2) array for a 2D one. If None, assume a 1D functional output.
             inputs: The inputs to display. If None, display all inputs.
             standardize: If True, standardize the indices between 0 and 1 for each output.
             title: The title of the plot. If None, no title is displayed.
@@ -394,28 +445,38 @@ class SensitivityAnalysis(object):
         Raises:
             NotImplementedError: If the dimension of the mesh is greater than 2.
         """
+        if isinstance(output, str):
+            output_name = output
+            output_component = 0
+        else:
+            output_name, output_component = output
+
         dataset = Dataset()
-        inputs_names = self._sort_and_filter_input_parameters((output, 0), inputs)
+        inputs_names = self._sort_and_filter_input_parameters(
+            (output_name, output_component), inputs
+        )
         if standardize:
             main_indices = self.standardize_indices(self.main_indices)
         else:
             main_indices = self.main_indices
 
         data = []
-        for name in inputs_names:
-            data.append([value[name] for value in main_indices[output[0]]])
+        for input_name in inputs_names:
+            data.append(
+                [main_index[input_name] for main_index in main_indices[output_name]]
+            )
 
         data = array(data)[:, :, 0]
-        dataset.set_from_array(data, [output], sizes={output: data.shape[1]})
+        dataset.set_from_array(data, [output_name], sizes={output_name: data.shape[1]})
         dataset.row_names = inputs_names
         mesh = linspace(0, 1, data.shape[1]) if mesh is None else mesh
         dataset.set_metadata("mesh", mesh)
         mesh_dimension = len(dataset.metadata["mesh"].shape)
         if mesh_dimension == 1:
-            plot = Curves(dataset)
+            plot = Curves(dataset, mesh="mesh", variable=output_name)
             plot.title = title
         elif mesh_dimension == 2:
-            plot = Surfaces(dataset)
+            plot = Surfaces(dataset, mesh="mesh", variable=output_name)
         else:
             raise NotImplementedError
 
@@ -426,26 +487,24 @@ class SensitivityAnalysis(object):
             file_name=file_name,
             file_format=file_format,
             directory_path=directory_path,
-            mesh="mesh",
-            variable=output,
             properties=properties,
         )
         return plot
 
     def plot_bar(
         self,
-        outputs,  # type: OutputsType
-        inputs=None,  # type: Optional[Iterable[str]]
-        standardize=False,  # type: bool
-        title=None,  # type: Optional[str]
-        save=True,  # type: bool
-        show=False,  # type: bool
-        file_path=None,  # type: Optional[Union[str,Path]]
-        directory_path=None,  # type: Optional[Union[str,Path]]
-        file_name=None,  # type: Optional[str]
-        file_format=None,  # type: Optional[str]
-        **options  # type:int
-    ):  # type: (...) -> BarPlot
+        outputs: OutputsType,
+        inputs: Iterable[str] | None = None,
+        standardize: bool = False,
+        title: str | None = None,
+        save: bool = True,
+        show: bool = False,
+        file_path: str | Path | None = None,
+        directory_path: str | Path | None = None,
+        file_name: str | None = None,
+        file_format: str | None = None,
+        **options: int,
+    ) -> BarPlot:
         """Plot the sensitivity indices on a bar chart.
 
         This method may consider one or more outputs,
@@ -497,9 +556,7 @@ class SensitivityAnalysis(object):
         for name in inputs_names:
             dataset.add_variable(name, vstack(data[name]))
 
-        dataset.row_names = [
-            "{}({})".format(output[0], output[1]) for output in outputs
-        ]
+        dataset.row_names = [f"{output[0]}({output[1]})" for output in outputs]
         plot = BarPlot(dataset)
         plot.title = title
         plot.execute(
@@ -509,26 +566,26 @@ class SensitivityAnalysis(object):
             file_name=file_name,
             file_format=file_format,
             directory_path=directory_path,
-            **options
+            **options,
         )
         return plot
 
     def plot_radar(
         self,
-        outputs,  # type: OutputsType
-        inputs=None,  # type: Optional[Iterable[str]]
-        standardize=False,  # type: bool
-        title=None,  # type: Optional[str]
-        save=True,  # type: bool
-        show=False,  # type: bool
-        file_path=None,  # type: Optional[Union[str,Path]]
-        directory_path=None,  # type: Optional[Union[str,Path]]
-        file_name=None,  # type: Optional[str]
-        file_format=None,  # type: Optional[str]
-        min_radius=None,  # type: Optional[float]
-        max_radius=None,  # type: Optional[float]
-        **options  # type:bool
-    ):  # type: (...) -> RadarChart
+        outputs: OutputsType,
+        inputs: Iterable[str] | None = None,
+        standardize: bool = False,
+        title: str | None = None,
+        save: bool = True,
+        show: bool = False,
+        file_path: str | Path | None = None,
+        directory_path: str | Path | None = None,
+        file_name: str | None = None,
+        file_format: str | None = None,
+        min_radius: float | None = None,
+        max_radius: float | None = None,
+        **options: bool,
+    ) -> RadarChart:
         """Plot the sensitivity indices on a radar chart.
 
         This method may consider one or more outputs,
@@ -587,9 +644,7 @@ class SensitivityAnalysis(object):
         for name in inputs_names:
             dataset.add_variable(name, vstack(data[name]))
 
-        dataset.row_names = [
-            "{}({})".format(output[0], output[1]) for output in outputs
-        ]
+        dataset.row_names = [f"{output[0]}({output[1]})" for output in outputs]
         plot = RadarChart(dataset)
         plot.title = title
         plot.rmin = min_radius
@@ -601,15 +656,15 @@ class SensitivityAnalysis(object):
             file_name=file_name,
             file_format=file_format,
             directory_path=directory_path,
-            **options
+            **options,
         )
         return plot
 
     @staticmethod
     def _filter_names(
-        names,  # type:Iterable[str],
-        names_to_keep,  # type:Iterable[str]
-    ):  # type: (...) -> List[str]
+        names: Iterable[str],
+        names_to_keep: Iterable[str],
+    ) -> list[str]:
         """Sort and filter the names.
 
         Args:
@@ -625,9 +680,9 @@ class SensitivityAnalysis(object):
 
     def _sort_and_filter_input_parameters(
         self,
-        output,  # type: Tuple[str,int]
-        inputs_to_keep,  # type: Iterable[str]
-    ):  # type: (...) -> List[str]
+        output: tuple[str, int],
+        inputs_to_keep: Iterable[str],
+    ) -> list[str]:
         """Sort and filter the input parameters.
 
         Args:
@@ -642,19 +697,19 @@ class SensitivityAnalysis(object):
 
     def plot_comparison(
         self,
-        indices,  # type: List[SensitivityAnalysis]
-        output,  # type: Union[str,Tuple[str,int]]
-        inputs=None,  # type: Optional[Iterable[str]]
-        title=None,  # type: Optional[str]
-        use_bar_plot=True,  # type: bool
-        save=True,  # type: bool
-        show=False,  # type: bool
-        file_path=None,  # type: Optional[Union[str,Path]]
-        directory_path=None,  # type: Optional[Union[str,Path]]
-        file_name=None,  # type: Optional[str]
-        file_format=None,  # type: Optional[str]
-        **options  # type:bool
-    ):  # type: (...) -> Union[BarPlot,RadarChart]
+        indices: list[SensitivityAnalysis],
+        output: str | tuple[str, int],
+        inputs: Iterable[str] | None = None,
+        title: str | None = None,
+        use_bar_plot: bool = True,
+        save: bool = True,
+        show: bool = False,
+        file_path: str | Path | None = None,
+        directory_path: str | Path | None = None,
+        file_name: str | None = None,
+        file_format: str | None = None,
+        **options: bool,
+    ) -> BarPlot | RadarChart:
         """Plot a comparison between the current sensitivity indices and other ones.
 
         This method allows to use either a bar chart (default option) or a radar one.
@@ -720,14 +775,14 @@ class SensitivityAnalysis(object):
 
     def _save_show_plot(
         self,
-        fig,  # type: Figure
-        save=True,  # type: bool
-        show=False,  # type: bool
-        file_path=None,  # type: Optional[Union[str,Path]]
-        directory_path=None,  # type: Optional[Union[str,Path]]
-        file_name=None,  # type: Optional[str]
-        file_format=None,  # type: Optional[str]
-    ):  # type: (...) -> Figure
+        fig: Figure,
+        save: bool = True,
+        show: bool = False,
+        file_path: str | Path | None = None,
+        directory_path: str | Path | None = None,
+        file_name: str | None = None,
+        file_format: str | None = None,
+    ) -> Figure:
         """Save or show the plot.
 
         Args:
@@ -761,7 +816,7 @@ class SensitivityAnalysis(object):
         save_show_figure(fig, show, file_path)
         return fig
 
-    def export_to_dataset(self):  # type: (...) -> Dataset
+    def export_to_dataset(self) -> Dataset:
         """Convert :attr:`.SensitivityAnalysis.indices` into a :class:`.Dataset`.
 
         Returns:
@@ -772,7 +827,7 @@ class SensitivityAnalysis(object):
         rows_names = []
         for input_name in self.inputs_names:
             for input_component in range(sizes[input_name]):
-                rows_names.append("{}({})".format(input_name, input_component))
+                rows_names.append(f"{input_name}({input_component})")
 
         dataset = Dataset(by_group=False)
         for method, indices in self.indices.items():
@@ -794,8 +849,8 @@ class SensitivityAnalysis(object):
 
     @staticmethod
     def standardize_indices(
-        indices,  # type: IndicesType
-    ):  # type: (...) -> IndicesType
+        indices: IndicesType,
+    ) -> IndicesType:
         """Standardize the sensitivity indices for each output component.
 
         Each index is replaced by its absolute value divided by the largest index.
@@ -811,7 +866,7 @@ class SensitivityAnalysis(object):
         for output_name, output_indices in indices.items():
             for output_component, output_component_indices in enumerate(output_indices):
                 max_value = max(
-                    [abs(value)[0] for value in output_component_indices.values()]
+                    abs(value)[0] for value in output_component_indices.values()
                 )
 
                 for input_name, input_indices in output_component_indices.items():
