@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2021 IRT Saint Exupéry, https://www.irt-saintexupery.com
 #
 # This program is free software; you can redistribute it and/or
@@ -13,13 +12,11 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-
 # Contributors:
 #    INITIAL AUTHORS - initial API and implementation and/or initial
 #                           documentation
 #        :author: Matthias De Lozzo
 #    OTHER AUTHORS   - MACROSCOPIC CHANGES
-
 """Class for the parametric estimation of statistics from a dataset.
 
 Overview
@@ -90,26 +87,28 @@ Additional ones are:
 - :meth:`.plot_criteria`:
   this method plots the criterion values for a given variable.
 """
-
-from __future__ import division, unicode_literals
+from __future__ import annotations
 
 import logging
 import os
-from typing import Dict, Iterable, Optional, Sequence, Tuple, Union
+from typing import Iterable
+from typing import Sequence
 
 import matplotlib.pyplot as plt
-from numpy import array, linspace, ndarray
+from numpy import array
+from numpy import linspace
+from numpy import ndarray
 
 from gemseo.core.dataset import Dataset
 from gemseo.third_party.prettytable.prettytable import PrettyTable
 from gemseo.uncertainty.distributions.openturns.distribution import OTDistribution
-from gemseo.uncertainty.distributions.openturns.fitting import (
-    MeasureType,
-    OTDistributionFitter,
-)
+from gemseo.uncertainty.distributions.openturns.fitting import MeasureType
+from gemseo.uncertainty.distributions.openturns.fitting import OTDistributionFitter
 from gemseo.uncertainty.statistics.statistics import Statistics
 from gemseo.uncertainty.statistics.tolerance_interval.distribution import (
     ToleranceIntervalFactory,
+)
+from gemseo.uncertainty.statistics.tolerance_interval.distribution import (
     ToleranceIntervalSide,
 )
 
@@ -119,18 +118,6 @@ LOGGER = logging.getLogger(__name__)
 class ParametricStatistics(Statistics):
     """Parametric estimation of statistics.
 
-    Attributes:
-        fitting_criterion (str): The name of the goodness-of-fit criterion,
-            measuring how the distribution fits the data.
-        level (float): The test level,
-            i.e. risk of committing a Type 1 error,
-            that is an incorrect rejection of a true null hypothesis,
-            for criteria based on test hypothesis.
-        selection_criterion (str): The name of the selection criterion
-            to select a distribution from a list of candidates.
-        distributions (dict(str, dict(str, OTDistribution))): The probability
-            distributions of the random variables.
-
     Examples:
         >>> from gemseo.api import (
         ...     create_discipline,
@@ -141,9 +128,8 @@ class ParametricStatistics(Statistics):
         >>>
         >>> expressions = {"y1": "x1+2*x2", "y2": "x1-3*x2"}
         >>> discipline = create_discipline(
-        ...     "AnalyticDiscipline", expressions_dict=expressions
+        ...     "AnalyticDiscipline", expressions=expressions
         ... )
-        >>> discipline.set_cache_policy(discipline.MEMORY_FULL_CACHE)
         >>>
         >>> parameter_space = create_parameter_space()
         >>> parameter_space.add_random_variable(
@@ -160,7 +146,7 @@ class ParametricStatistics(Statistics):
         ... )
         >>> scenario.execute({'algo': 'OT_MONTE_CARLO', 'n_samples': 100})
         >>>
-        >>> dataset = discipline.cache.export_to_dataset()
+        >>> dataset = scenario.export_to_dataset(opt_naming=False)
         >>>
         >>> statistics = ParametricStatistics(
         ...     dataset, ['Normal', 'Uniform', 'Triangular']
@@ -169,6 +155,23 @@ class ParametricStatistics(Statistics):
         >>> mean = statistics.mean()
     """
 
+    fitting_criterion: str
+    """The name of the goodness-of-fit criterion,
+    measuring how the distribution fits the data."""
+
+    level: float
+    """The test level,
+    i.e. risk of committing a Type 1 error,
+    that is an incorrect rejection of a true null hypothesis,
+    for criteria based on test hypothesis."""
+
+    selection_criterion: str
+    """The name of the selection criterion
+    to select a distribution from a list of candidates."""
+
+    distributions: dict[str, dict[str, OTDistribution]]
+    """The probability distributions of the random variables."""
+
     AVAILABLE_DISTRIBUTIONS = sorted(
         OTDistributionFitter._AVAILABLE_DISTRIBUTIONS.keys()
     )
@@ -178,15 +181,15 @@ class ParametricStatistics(Statistics):
 
     def __init__(
         self,
-        dataset,  # type: Dataset
-        distributions,  # type: Sequence[str]
-        variables_names=None,  # type: Optional[Iterable[str]]
-        fitting_criterion="BIC",  # type: str
-        level=0.05,  # type: float
-        selection_criterion="best",  # type: str
-        name=None,  # type: Optional[str]
-    ):  # type: (...) -> None  # noqa: D205,D212,D415
-        """
+        dataset: Dataset,
+        distributions: Sequence[str],
+        variables_names: Iterable[str] | None = None,
+        fitting_criterion: str = "BIC",
+        level: float = 0.05,
+        selection_criterion: str = "best",
+        name: str | None = None,
+    ) -> None:
+        """.. # noqa: D205,D212,D415
         Args:
             distributions: The names of the distributions.
             fitting_criterion: The name of
@@ -202,7 +205,7 @@ class ParametricStatistics(Statistics):
                 to select a distribution from a list of candidates.
                 Either 'first' or 'best'.
         """
-        super(ParametricStatistics, self).__init__(dataset, variables_names, name)
+        super().__init__(dataset, variables_names, name)
         significance_tests = OTDistributionFitter.SIGNIFICANCE_TESTS
         self.fitting_criterion = fitting_criterion
         self.selection_criterion = selection_criterion
@@ -218,8 +221,8 @@ class ParametricStatistics(Statistics):
 
     def _build_distributions(
         self,
-        distributions,  # type: Sequence[str]
-    ):  # type: (...) -> None
+        distributions: Sequence[str],
+    ) -> None:
         """Build distributions from distributions names.
 
         Args:
@@ -228,7 +231,7 @@ class ParametricStatistics(Statistics):
         self._all_distributions = self._fit_distributions(distributions)
         self.distributions = self._select_best_distributions(distributions)
 
-    def get_fitting_matrix(self):  # type: (...) -> str
+    def get_fitting_matrix(self) -> str:
         """Get the fitting matrix.
 
         This matrix contains goodness-of-fit measures
@@ -249,8 +252,8 @@ class ParametricStatistics(Statistics):
 
     def get_criteria(
         self,
-        variable,  # type:str
-    ):  # type: (...) -> Tuple[Dict[str,float],bool]
+        variable: str,
+    ) -> tuple[dict[str, float], bool]:
         """Get criteria for a given variable name and the different distributions.
 
         Args:
@@ -277,13 +280,13 @@ class ParametricStatistics(Statistics):
 
     def plot_criteria(
         self,
-        variable,  # type: str
-        title=None,  # type: Optional[str]
-        save=False,  # type:bool
-        show=True,  # type: bool
-        n_legend_cols=4,  # type: int
-        directory=".",  # type:str
-    ):  # type: (...) -> None
+        variable: str,
+        title: str | None = None,
+        save: bool = False,
+        show: bool = True,
+        n_legend_cols: int = 4,
+        directory: str = ".",
+    ) -> None:
         """Plot criteria for a given variable name.
 
         Args:
@@ -315,7 +318,7 @@ class ParametricStatistics(Statistics):
         plt.subplot(121)
         plt.bar(x_values, y_values, tick_label=labels, align="center")
         if is_p_value:
-            plt.ylabel("p-value from {} test".format(self.fitting_criterion))
+            plt.ylabel(f"p-value from {self.fitting_criterion} test")
             plt.axhline(self.level, color="r", linewidth=2.0)
         plt.grid(True, "both")
         plt.subplot(122)
@@ -350,8 +353,8 @@ class ParametricStatistics(Statistics):
         plt.close()
 
     def _select_best_distributions(
-        self, distributions_names  # type: Sequence[str]
-    ):  # type: (...) -> Dict[str,Dict[str,Union[str,OTDistribution]]]
+        self, distributions_names: Sequence[str]
+    ) -> dict[str, dict[str, str | OTDistribution]]:
         """Select the best distributions for the different variables.
 
         Args:
@@ -380,21 +383,20 @@ class ParametricStatistics(Statistics):
 
     def _fit_distributions(
         self,
-        distributions,  # type: Iterable[str]
-    ):  # type: (...) -> Dict[str,Dict[str,Dict[str,Union[OTDistribution,MeasureType]]]]
+        distributions: Iterable[str],
+    ) -> dict[str, dict[str, dict[str, OTDistribution | MeasureType]]]:
         """Fit different distributions for the different marginals.
 
         Args:
             distributions: The distributions names.
 
         Returns:
-            dict(str, dict): The distributions for the different variables.
+            The distributions for the different variables.
         """
-        dist_list = ", ".join(distributions)
         LOGGER.info(
             "Fit different distributions (%s) per variable "
             "and compute the goodness-of-fit criterion.",
-            dist_list,
+            ", ".join(distributions),
         )
         results = {}
         for variable in self.names:
@@ -407,10 +409,10 @@ class ParametricStatistics(Statistics):
 
     def _fit_marginal_distributions(
         self,
-        variable,  # type: str
-        sample,  # type: ndarray
-        distributions,  # type: Iterable[str]
-    ):  # type: (...) -> Dict[str,Dict[str,Union[OTDistribution,MeasureType]]]
+        variable: str,
+        sample: ndarray,
+        distributions: Iterable[str],
+    ) -> dict[str, dict[str, OTDistribution | MeasureType]]:
         """Fit different distributions for a given dataset marginal.
 
         Args:
@@ -433,29 +435,29 @@ class ParametricStatistics(Statistics):
             result[distribution]["criterion"] = test_result
         return result
 
-    def compute_maximum(self):  # type: (...) -> Dict[str, ndarray]  # noqa: D102
+    def compute_maximum(self) -> dict[str, ndarray]:  # noqa: D102
         result = {
             name: self.distributions[name]["value"].math_upper_bound
             for name in self.names
         }
         return result
 
-    def compute_mean(self):  # type: (...) -> Dict[str, ndarray]  # noqa: D102
+    def compute_mean(self) -> dict[str, ndarray]:  # noqa: D102
         result = {name: self.distributions[name]["value"].mean for name in self.names}
         return result
 
-    def compute_minimum(self):  # type: (...) -> Dict[str, ndarray] # noqa: D102
+    def compute_minimum(self) -> dict[str, ndarray]:  # noqa: D102
         result = {
             name: self.distributions[name]["value"].math_lower_bound
             for name in self.names
         }
         return result
 
-    def compute_probability(
+    def compute_probability(  # noqa: D102
         self,
-        thresh,  # type: float
-        greater=True,  # type: bool
-    ):  # type: (...) -> Dict[str, ndarray] # noqa: D102
+        thresh: float,
+        greater: bool = True,
+    ) -> dict[str, ndarray]:
         dist = self.distributions
         if greater:
             result = {
@@ -469,13 +471,13 @@ class ParametricStatistics(Statistics):
             }
         return result
 
-    def compute_tolerance_interval(
+    def compute_tolerance_interval(  # noqa: D102
         self,
-        coverage,  # type: float
-        confidence=0.95,  # type: float
-        side=ToleranceIntervalSide.BOTH,  # type: ToleranceIntervalSide
-    ):  # type: (...) -> Dict[str, Tuple[ndarray,ndarray]]
-        # noqa: D102 D205 D212 D415
+        coverage: float,
+        confidence: float = 0.95,
+        side: ToleranceIntervalSide = ToleranceIntervalSide.BOTH,
+    ) -> dict[str, tuple[ndarray, ndarray]]:
+
         if not 0.0 <= coverage <= 1.0:
             raise ValueError("The argument 'coverage' must be number in [0,1].")
         if not 0.0 <= confidence <= 1.0:
@@ -490,10 +492,10 @@ class ParametricStatistics(Statistics):
             limits[variable] = tolerance_interval.compute(coverage, confidence, side)
         return limits
 
-    def compute_quantile(
+    def compute_quantile(  # noqa: D102
         self,
-        prob,  # type:float
-    ):  # type: (...) -> Dict[str,ndarray] # noqa: D102
+        prob: float,
+    ) -> dict[str, ndarray]:
         prob = array([prob])
         result = {
             name: self.distributions[name]["value"].compute_inverse_cdf(prob)
@@ -501,33 +503,33 @@ class ParametricStatistics(Statistics):
         }
         return result
 
-    def compute_standard_deviation(
+    def compute_standard_deviation(  # noqa: D102
         self,
-    ):  # type: (...) -> Dict[str, ndarray]  # noqa: D102
+    ) -> dict[str, ndarray]:
         result = {
             name: self.distributions[name]["value"].standard_deviation
             for name in self.names
         }
         return result
 
-    def compute_variance(self):  # type: (...) -> Dict[str, ndarray]  # noqa: D102
+    def compute_variance(self) -> dict[str, ndarray]:  # noqa: D102
         result = {
             name: self.distributions[name]["value"].standard_deviation ** 2
             for name in self.names
         }
         return result
 
-    def compute_moment(
+    def compute_moment(  # noqa: D102
         self,
-        order,  # type: int
-    ):  # type: (...) -> Dict[str, ndarray]  # noqa: D102
+        order: int,
+    ) -> dict[str, ndarray]:
         dist = self.distributions
         result = [
             dist[name]["value"].distribution.getMoment(order)[0] for name in self.names
         ]
         return result
 
-    def compute_range(self):  # type: (...) -> Dict[str, ndarray]  # noqa: D102
+    def compute_range(self) -> dict[str, ndarray]:  # noqa: D102
         result = {}
         for name in self.names:
             dist = self.distributions[name]["value"]
