@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2021 IRT Saint Exupéry, https://www.irt-saintexupery.com
 #
 # This program is free software; you can redistribute it and/or
@@ -14,7 +13,7 @@
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 """Compatibility between different versions of openturns."""
-from __future__ import absolute_import
+from __future__ import annotations
 
 import openturns
 from packaging import version
@@ -22,14 +21,13 @@ from packaging import version
 if version.parse(openturns.__version__) < version.parse("1.18"):
 
     def get_eigenvalues(
-        result,  # type: openturns.KarhunenLoeveResult
-    ):  # type: (...) -> openturns.Point
+        result: openturns.KarhunenLoeveResult,
+    ) -> openturns.Point:
         return result.getEigenValues()
 
-
 else:
 
     def get_eigenvalues(
-        result,  # type: openturns.KarhunenLoeveResult
-    ):  # type: (...) -> openturns.Point
+        result: openturns.KarhunenLoeveResult,
+    ) -> openturns.Point:
         return result.getEigenvalues()
