@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2021 IRT Saint Exupéry, https://www.irt-saintexupery.com
 #
 # This program is free software; you can redistribute it and/or
@@ -13,21 +12,18 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-
 # Contributors:
 #    INITIAL AUTHORS - initial API and implementation and/or
 #                      initial documentation
 #        :author:  Francois Gallard
 #    OTHER AUTHORS   - MACROSCOPIC CHANGES
-"""
-Singletons implementation and variants
-**************************************
-"""
-from __future__ import division, unicode_literals
+"""Singletons implementation and variants."""
+from __future__ import annotations
 
 from os.path import realpath
+from typing import Any
 
-from gemseo.utils.py23_compat import string_types
+from six import with_metaclass
 
 
 class SingleInstancePerAttributeId(type):
@@ -59,28 +55,45 @@ class SingleInstancePerAttributeId(type):
         return inst
 
 
-class SingleInstancePerAttributeEq(type):
-    """A Singleton-like design pattern so that subclasses are only instantiated when the
-    discipline instance passed as input of the constructor is different from already
-    created instances.
+class _Multiton(type):
+    """A metaclass for implementing the Multiton design pattern.
 
-    The test if the instances are equal is made with the obj1 == obj2 operator
+    See `Multiton <https://en.wikipedia.org/wiki/Multiton_pattern>`.
+
+    As opposed to the functools.lru_cache,
+    the objects built from this metaclass can be pickled.
+
+    .. warning:
+
+        Like the standard functools.lru_cache,
+        the kwargs order is not preserved:
+        it means that f(x=1, y=2) is treated as a
+        distinct call from f(y=2, x=1) which will be cached separately.
     """
 
-    instances = {}
+    _cache: Any = {}
 
-    # Eclipse is not happy with "cls" as first
-    # argument but this is an eclipse bug.
-    # "function.MDOFunctionGenerator' should have self as first parameter"
-    def __call__(cls, *args, **kwargs):
-        if not args:
-            args = [None]
-        inst_key = (id(cls),) + tuple(args)
-        inst = cls.instances.get(inst_key)
-        if inst is None:
+    def __call__(
+        cls,
+        *args: Any,
+        **kwargs: Any,
+    ) -> None:
+        key = (cls,) + args + tuple(kwargs.items())
+        try:
+            return cls._cache[key]
+        except KeyError:
             inst = type.__call__(cls, *args, **kwargs)
-            cls.instances[inst_key] = inst
-        return inst
+            cls._cache[key] = inst
+            return inst
+
+    @classmethod
+    def cache_clear(cls) -> None:
+        """Clear the cache."""
+        cls._cache = {}
+
+
+# Provide a naturally derivable class.
+Multiton = with_metaclass(_Multiton, object)
 
 
 class SingleInstancePerFileAttribute(type):
@@ -103,7 +116,7 @@ class SingleInstancePerFileAttribute(type):
             )
         fpath = args[0]
 
-        if not isinstance(fpath, string_types):
+        if not isinstance(fpath, str):
             raise TypeError(
                 "Argument 0 is not a string but of type :" + str(type(fpath))
             )
