@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2021 IRT Saint Exupéry, https://www.irt-saintexupery.com
 #
 # This program is free software; you can redistribute it and/or
@@ -13,19 +12,13 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-
 # Contributors:
 #    INITIAL AUTHORS - API and implementation and/or documentation
 #        :author: Matthias De Lozzo
 #    OTHER AUTHORS   - MACROSCOPIC CHANGES
-from __future__ import division, unicode_literals
-
 import pytest
-
-from gemseo.problems.scalable.data_driven.api import (
-    create_scalability_study,
-    plot_scalability_results,
-)
+from gemseo.problems.scalable.data_driven.api import create_scalability_study
+from gemseo.problems.scalable.data_driven.api import plot_scalability_results
 
 
 def test_create_scalability_study():
